"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2019, OpenTelemetry Authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import opentelemetry.proto.common.v1.common_pb2
import opentelemetry.proto.resource.v1.resource_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _SpanFlags:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _SpanFlagsEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[
        _SpanFlags.ValueType
    ],
    builtins.type,
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SPAN_FLAGS_DO_NOT_USE: _SpanFlags.ValueType  # 0
    """The zero value for the enum. Should not be used for comparisons.
    Instead use bitwise "and" with the appropriate mask as shown above.
    """
    SPAN_FLAGS_TRACE_FLAGS_MASK: _SpanFlags.ValueType  # 255
    """Bits 0-7 are used for trace flags."""
    SPAN_FLAGS_CONTEXT_HAS_IS_REMOTE_MASK: _SpanFlags.ValueType  # 256
    """Bits 8 and 9 are used to indicate that the parent span or link span is remote.
    Bit 8 (`HAS_IS_REMOTE`) indicates whether the value is known.
    Bit 9 (`IS_REMOTE`) indicates whether the span or link is remote.
    """
    SPAN_FLAGS_CONTEXT_IS_REMOTE_MASK: _SpanFlags.ValueType  # 512

class SpanFlags(_SpanFlags, metaclass=_SpanFlagsEnumTypeWrapper):
    """SpanFlags represents constants used to interpret the
    Span.flags field, which is protobuf 'fixed32' type and is to
    be used as bit-fields. Each non-zero value defined in this enum is
    a bit-mask.  To extract the bit-field, for example, use an
    expression like:

      (span.flags & SPAN_FLAGS_TRACE_FLAGS_MASK)

    See https://www.w3.org/TR/trace-context-2/#trace-flags for the flag definitions.

    Note that Span flags were introduced in version 1.1 of the
    OpenTelemetry protocol.  Older Span producers do not set this
    field, consequently consumers should not rely on the absence of a
    particular flag bit to indicate the presence of a particular feature.
    """

SPAN_FLAGS_DO_NOT_USE: SpanFlags.ValueType  # 0
"""The zero value for the enum. Should not be used for comparisons.
Instead use bitwise "and" with the appropriate mask as shown above.
"""
SPAN_FLAGS_TRACE_FLAGS_MASK: SpanFlags.ValueType  # 255
"""Bits 0-7 are used for trace flags."""
SPAN_FLAGS_CONTEXT_HAS_IS_REMOTE_MASK: SpanFlags.ValueType  # 256
"""Bits 8 and 9 are used to indicate that the parent span or link span is remote.
Bit 8 (`HAS_IS_REMOTE`) indicates whether the value is known.
Bit 9 (`IS_REMOTE`) indicates whether the span or link is remote.
"""
SPAN_FLAGS_CONTEXT_IS_REMOTE_MASK: SpanFlags.ValueType  # 512
global___SpanFlags = SpanFlags

@typing_extensions.final
class TracesData(google.protobuf.message.Message):
    """TracesData represents the traces data that can be stored in a persistent storage,
    OR can be embedded by other protocols that transfer OTLP traces data but do
    not implement the OTLP protocol.

    The main difference between this message and collector protocol is that
    in this message there will not be any "control" or "metadata" specific to
    OTLP protocol.

    When new fields are added into this message, the OTLP request MUST be updated
    as well.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESOURCE_SPANS_FIELD_NUMBER: builtins.int
    @property
    def resource_spans(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        global___ResourceSpans
    ]:
        """An array of ResourceSpans.
        For data coming from a single resource this array will typically contain
        one element. Intermediary nodes that receive data from multiple origins
        typically batch the data before forwarding further and in that case this
        array will contain multiple elements.
        """

    def __init__(
        self,
        *,
        resource_spans: (
            collections.abc.Iterable[global___ResourceSpans] | None
        ) = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "resource_spans", b"resource_spans"
        ],
    ) -> None: ...

global___TracesData = TracesData

@typing_extensions.final
class ResourceSpans(google.protobuf.message.Message):
    """A collection of ScopeSpans from a Resource."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESOURCE_FIELD_NUMBER: builtins.int
    SCOPE_SPANS_FIELD_NUMBER: builtins.int
    SCHEMA_URL_FIELD_NUMBER: builtins.int
    @property
    def resource(
        self,
    ) -> opentelemetry.proto.resource.v1.resource_pb2.Resource:
        """The resource for the spans in this message.
        If this field is not set then no resource info is known.
        """

    @property
    def scope_spans(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        global___ScopeSpans
    ]:
        """A list of ScopeSpans that originate from a resource."""
    schema_url: builtins.str
    """The Schema URL, if known. This is the identifier of the Schema that the resource data
    is recorded in. To learn more about Schema URL see
    https://opentelemetry.io/docs/specs/otel/schemas/#schema-url
    This schema_url applies to the data in the "resource" field. It does not apply
    to the data in the "scope_spans" field which have their own schema_url field.
    """
    def __init__(
        self,
        *,
        resource: (
            opentelemetry.proto.resource.v1.resource_pb2.Resource | None
        ) = ...,
        scope_spans: (
            collections.abc.Iterable[global___ScopeSpans] | None
        ) = ...,
        schema_url: builtins.str = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["resource", b"resource"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "resource",
            b"resource",
            "schema_url",
            b"schema_url",
            "scope_spans",
            b"scope_spans",
        ],
    ) -> None: ...

global___ResourceSpans = ResourceSpans

@typing_extensions.final
class ScopeSpans(google.protobuf.message.Message):
    """A collection of Spans produced by an InstrumentationScope."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SCOPE_FIELD_NUMBER: builtins.int
    SPANS_FIELD_NUMBER: builtins.int
    SCHEMA_URL_FIELD_NUMBER: builtins.int
    @property
    def scope(
        self,
    ) -> opentelemetry.proto.common.v1.common_pb2.InstrumentationScope:
        """The instrumentation scope information for the spans in this message.
        Semantically when InstrumentationScope isn't set, it is equivalent with
        an empty instrumentation scope name (unknown).
        """

    @property
    def spans(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        global___Span
    ]:
        """A list of Spans that originate from an instrumentation scope."""
    schema_url: builtins.str
    """The Schema URL, if known. This is the identifier of the Schema that the span data
    is recorded in. To learn more about Schema URL see
    https://opentelemetry.io/docs/specs/otel/schemas/#schema-url
    This schema_url applies to all spans and span events in the "spans" field.
    """
    def __init__(
        self,
        *,
        scope: (
            opentelemetry.proto.common.v1.common_pb2.InstrumentationScope
            | None
        ) = ...,
        spans: collections.abc.Iterable[global___Span] | None = ...,
        schema_url: builtins.str = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["scope", b"scope"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "schema_url", b"schema_url", "scope", b"scope", "spans", b"spans"
        ],
    ) -> None: ...

global___ScopeSpans = ScopeSpans

@typing_extensions.final
class Span(google.protobuf.message.Message):
    """A Span represents a single operation performed by a single component of the system.

    The next available field id is 17.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _SpanKind:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _SpanKindEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[
            Span._SpanKind.ValueType
        ],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        SPAN_KIND_UNSPECIFIED: Span._SpanKind.ValueType  # 0
        """Unspecified. Do NOT use as default.
        Implementations MAY assume SpanKind to be INTERNAL when receiving UNSPECIFIED.
        """
        SPAN_KIND_INTERNAL: Span._SpanKind.ValueType  # 1
        """Indicates that the span represents an internal operation within an application,
        as opposed to an operation happening at the boundaries. Default value.
        """
        SPAN_KIND_SERVER: Span._SpanKind.ValueType  # 2
        """Indicates that the span covers server-side handling of an RPC or other
        remote network request.
        """
        SPAN_KIND_CLIENT: Span._SpanKind.ValueType  # 3
        """Indicates that the span describes a request to some remote service."""
        SPAN_KIND_PRODUCER: Span._SpanKind.ValueType  # 4
        """Indicates that the span describes a producer sending a message to a broker.
        Unlike CLIENT and SERVER, there is often no direct critical path latency relationship
        between producer and consumer spans. A PRODUCER span ends when the message was accepted
        by the broker while the logical processing of the message might span a much longer time.
        """
        SPAN_KIND_CONSUMER: Span._SpanKind.ValueType  # 5
        """Indicates that the span describes consumer receiving a message from a broker.
        Like the PRODUCER kind, there is often no direct critical path latency relationship
        between producer and consumer spans.
        """

    class SpanKind(_SpanKind, metaclass=_SpanKindEnumTypeWrapper):
        """SpanKind is the type of span. Can be used to specify additional relationships between spans
        in addition to a parent/child relationship.
        """

    SPAN_KIND_UNSPECIFIED: Span.SpanKind.ValueType  # 0
    """Unspecified. Do NOT use as default.
    Implementations MAY assume SpanKind to be INTERNAL when receiving UNSPECIFIED.
    """
    SPAN_KIND_INTERNAL: Span.SpanKind.ValueType  # 1
    """Indicates that the span represents an internal operation within an application,
    as opposed to an operation happening at the boundaries. Default value.
    """
    SPAN_KIND_SERVER: Span.SpanKind.ValueType  # 2
    """Indicates that the span covers server-side handling of an RPC or other
    remote network request.
    """
    SPAN_KIND_CLIENT: Span.SpanKind.ValueType  # 3
    """Indicates that the span describes a request to some remote service."""
    SPAN_KIND_PRODUCER: Span.SpanKind.ValueType  # 4
    """Indicates that the span describes a producer sending a message to a broker.
    Unlike CLIENT and SERVER, there is often no direct critical path latency relationship
    between producer and consumer spans. A PRODUCER span ends when the message was accepted
    by the broker while the logical processing of the message might span a much longer time.
    """
    SPAN_KIND_CONSUMER: Span.SpanKind.ValueType  # 5
    """Indicates that the span describes consumer receiving a message from a broker.
    Like the PRODUCER kind, there is often no direct critical path latency relationship
    between producer and consumer spans.
    """

    @typing_extensions.final
    class Event(google.protobuf.message.Message):
        """Event is a time-stamped annotation of the span, consisting of user-supplied
        text description and key-value pairs.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TIME_UNIX_NANO_FIELD_NUMBER: builtins.int
        NAME_FIELD_NUMBER: builtins.int
        ATTRIBUTES_FIELD_NUMBER: builtins.int
        DROPPED_ATTRIBUTES_COUNT_FIELD_NUMBER: builtins.int
        time_unix_nano: builtins.int
        """time_unix_nano is the time the event occurred."""
        name: builtins.str
        """name of the event.
        This field is semantically required to be set to non-empty string.
        """
        @property
        def attributes(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
            opentelemetry.proto.common.v1.common_pb2.KeyValue
        ]:
            """attributes is a collection of attribute key/value pairs on the event.
            Attribute keys MUST be unique (it is not allowed to have more than one
            attribute with the same key).
            """
        dropped_attributes_count: builtins.int
        """dropped_attributes_count is the number of dropped attributes. If the value is 0,
        then no attributes were dropped.
        """
        def __init__(
            self,
            *,
            time_unix_nano: builtins.int = ...,
            name: builtins.str = ...,
            attributes: (
                collections.abc.Iterable[
                    opentelemetry.proto.common.v1.common_pb2.KeyValue
                ]
                | None
            ) = ...,
            dropped_attributes_count: builtins.int = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing_extensions.Literal[
                "attributes",
                b"attributes",
                "dropped_attributes_count",
                b"dropped_attributes_count",
                "name",
                b"name",
                "time_unix_nano",
                b"time_unix_nano",
            ],
        ) -> None: ...

    @typing_extensions.final
    class Link(google.protobuf.message.Message):
        """A pointer from the current span to another span in the same trace or in a
        different trace. For example, this can be used in batching operations,
        where a single batch handler processes multiple requests from different
        traces or when the handler receives a request from a different project.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TRACE_ID_FIELD_NUMBER: builtins.int
        SPAN_ID_FIELD_NUMBER: builtins.int
        TRACE_STATE_FIELD_NUMBER: builtins.int
        ATTRIBUTES_FIELD_NUMBER: builtins.int
        DROPPED_ATTRIBUTES_COUNT_FIELD_NUMBER: builtins.int
        FLAGS_FIELD_NUMBER: builtins.int
        trace_id: builtins.bytes
        """A unique identifier of a trace that this linked span is part of. The ID is a
        16-byte array.
        """
        span_id: builtins.bytes
        """A unique identifier for the linked span. The ID is an 8-byte array."""
        trace_state: builtins.str
        """The trace_state associated with the link."""
        @property
        def attributes(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
            opentelemetry.proto.common.v1.common_pb2.KeyValue
        ]:
            """attributes is a collection of attribute key/value pairs on the link.
            Attribute keys MUST be unique (it is not allowed to have more than one
            attribute with the same key).
            """
        dropped_attributes_count: builtins.int
        """dropped_attributes_count is the number of dropped attributes. If the value is 0,
        then no attributes were dropped.
        """
        flags: builtins.int
        """Flags, a bit field.

        Bits 0-7 (8 least significant bits) are the trace flags as defined in W3C Trace
        Context specification. To read the 8-bit W3C trace flag, use
        `flags & SPAN_FLAGS_TRACE_FLAGS_MASK`.

        See https://www.w3.org/TR/trace-context-2/#trace-flags for the flag definitions.

        Bits 8 and 9 represent the 3 states of whether the link is remote.
        The states are (unknown, is not remote, is remote).
        To read whether the value is known, use `(flags & SPAN_FLAGS_CONTEXT_HAS_IS_REMOTE_MASK) != 0`.
        To read whether the link is remote, use `(flags & SPAN_FLAGS_CONTEXT_IS_REMOTE_MASK) != 0`.

        Readers MUST NOT assume that bits 10-31 (22 most significant bits) will be zero.
        When creating new spans, bits 10-31 (most-significant 22-bits) MUST be zero.

        [Optional].
        """
        def __init__(
            self,
            *,
            trace_id: builtins.bytes = ...,
            span_id: builtins.bytes = ...,
            trace_state: builtins.str = ...,
            attributes: (
                collections.abc.Iterable[
                    opentelemetry.proto.common.v1.common_pb2.KeyValue
                ]
                | None
            ) = ...,
            dropped_attributes_count: builtins.int = ...,
            flags: builtins.int = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing_extensions.Literal[
                "attributes",
                b"attributes",
                "dropped_attributes_count",
                b"dropped_attributes_count",
                "flags",
                b"flags",
                "span_id",
                b"span_id",
                "trace_id",
                b"trace_id",
                "trace_state",
                b"trace_state",
            ],
        ) -> None: ...

    TRACE_ID_FIELD_NUMBER: builtins.int
    SPAN_ID_FIELD_NUMBER: builtins.int
    TRACE_STATE_FIELD_NUMBER: builtins.int
    PARENT_SPAN_ID_FIELD_NUMBER: builtins.int
    FLAGS_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    KIND_FIELD_NUMBER: builtins.int
    START_TIME_UNIX_NANO_FIELD_NUMBER: builtins.int
    END_TIME_UNIX_NANO_FIELD_NUMBER: builtins.int
    ATTRIBUTES_FIELD_NUMBER: builtins.int
    DROPPED_ATTRIBUTES_COUNT_FIELD_NUMBER: builtins.int
    EVENTS_FIELD_NUMBER: builtins.int
    DROPPED_EVENTS_COUNT_FIELD_NUMBER: builtins.int
    LINKS_FIELD_NUMBER: builtins.int
    DROPPED_LINKS_COUNT_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    trace_id: builtins.bytes
    """A unique identifier for a trace. All spans from the same trace share
    the same `trace_id`. The ID is a 16-byte array. An ID with all zeroes OR
    of length other than 16 bytes is considered invalid (empty string in OTLP/JSON
    is zero-length and thus is also invalid).

    This field is required.
    """
    span_id: builtins.bytes
    """A unique identifier for a span within a trace, assigned when the span
    is created. The ID is an 8-byte array. An ID with all zeroes OR of length
    other than 8 bytes is considered invalid (empty string in OTLP/JSON
    is zero-length and thus is also invalid).

    This field is required.
    """
    trace_state: builtins.str
    """trace_state conveys information about request position in multiple distributed tracing graphs.
    It is a trace_state in w3c-trace-context format: https://www.w3.org/TR/trace-context/#tracestate-header
    See also https://github.com/w3c/distributed-tracing for more details about this field.
    """
    parent_span_id: builtins.bytes
    """The `span_id` of this span's parent span. If this is a root span, then this
    field must be empty. The ID is an 8-byte array.
    """
    flags: builtins.int
    """Flags, a bit field.

    Bits 0-7 (8 least significant bits) are the trace flags as defined in W3C Trace
    Context specification. To read the 8-bit W3C trace flag, use
    `flags & SPAN_FLAGS_TRACE_FLAGS_MASK`.

    See https://www.w3.org/TR/trace-context-2/#trace-flags for the flag definitions.

    Bits 8 and 9 represent the 3 states of whether a span's parent
    is remote. The states are (unknown, is not remote, is remote).
    To read whether the value is known, use `(flags & SPAN_FLAGS_CONTEXT_HAS_IS_REMOTE_MASK) != 0`.
    To read whether the span is remote, use `(flags & SPAN_FLAGS_CONTEXT_IS_REMOTE_MASK) != 0`.

    When creating span messages, if the message is logically forwarded from another source
    with an equivalent flags fields (i.e., usually another OTLP span message), the field SHOULD
    be copied as-is. If creating from a source that does not have an equivalent flags field
    (such as a runtime representation of an OpenTelemetry span), the high 22 bits MUST
    be set to zero.
    Readers MUST NOT assume that bits 10-31 (22 most significant bits) will be zero.

    [Optional].
    """
    name: builtins.str
    """A description of the span's operation.

    For example, the name can be a qualified method name or a file name
    and a line number where the operation is called. A best practice is to use
    the same display name at the same call point in an application.
    This makes it easier to correlate spans in different traces.

    This field is semantically required to be set to non-empty string.
    Empty value is equivalent to an unknown span name.

    This field is required.
    """
    kind: global___Span.SpanKind.ValueType
    """Distinguishes between spans generated in a particular context. For example,
    two spans with the same name may be distinguished using `CLIENT` (caller)
    and `SERVER` (callee) to identify queueing latency associated with the span.
    """
    start_time_unix_nano: builtins.int
    """start_time_unix_nano is the start time of the span. On the client side, this is the time
    kept by the local machine where the span execution starts. On the server side, this
    is the time when the server's application handler starts running.
    Value is UNIX Epoch time in nanoseconds since 00:00:00 UTC on 1 January 1970.

    This field is semantically required and it is expected that end_time >= start_time.
    """
    end_time_unix_nano: builtins.int
    """end_time_unix_nano is the end time of the span. On the client side, this is the time
    kept by the local machine where the span execution ends. On the server side, this
    is the time when the server application handler stops running.
    Value is UNIX Epoch time in nanoseconds since 00:00:00 UTC on 1 January 1970.

    This field is semantically required and it is expected that end_time >= start_time.
    """
    @property
    def attributes(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        opentelemetry.proto.common.v1.common_pb2.KeyValue
    ]:
        """attributes is a collection of key/value pairs. Note, global attributes
        like server name can be set using the resource API. Examples of attributes:

            "/http/user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/71.0.3578.98 Safari/537.36"
            "/http/server_latency": 300
            "example.com/myattribute": true
            "example.com/score": 10.239

        The OpenTelemetry API specification further restricts the allowed value types:
        https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/common/README.md#attribute
        Attribute keys MUST be unique (it is not allowed to have more than one
        attribute with the same key).
        """
    dropped_attributes_count: builtins.int
    """dropped_attributes_count is the number of attributes that were discarded. Attributes
    can be discarded because their keys are too long or because there are too many
    attributes. If this value is 0, then no attributes were dropped.
    """
    @property
    def events(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        global___Span.Event
    ]:
        """events is a collection of Event items."""
    dropped_events_count: builtins.int
    """dropped_events_count is the number of dropped events. If the value is 0, then no
    events were dropped.
    """
    @property
    def links(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        global___Span.Link
    ]:
        """links is a collection of Links, which are references from this span to a span
        in the same or different trace.
        """
    dropped_links_count: builtins.int
    """dropped_links_count is the number of dropped links after the maximum size was
    enforced. If this value is 0, then no links were dropped.
    """
    @property
    def status(self) -> global___Status:
        """An optional final status for this span. Semantically when Status isn't set, it means
        span's status code is unset, i.e. assume STATUS_CODE_UNSET (code = 0).
        """

    def __init__(
        self,
        *,
        trace_id: builtins.bytes = ...,
        span_id: builtins.bytes = ...,
        trace_state: builtins.str = ...,
        parent_span_id: builtins.bytes = ...,
        flags: builtins.int = ...,
        name: builtins.str = ...,
        kind: global___Span.SpanKind.ValueType = ...,
        start_time_unix_nano: builtins.int = ...,
        end_time_unix_nano: builtins.int = ...,
        attributes: (
            collections.abc.Iterable[
                opentelemetry.proto.common.v1.common_pb2.KeyValue
            ]
            | None
        ) = ...,
        dropped_attributes_count: builtins.int = ...,
        events: collections.abc.Iterable[global___Span.Event] | None = ...,
        dropped_events_count: builtins.int = ...,
        links: collections.abc.Iterable[global___Span.Link] | None = ...,
        dropped_links_count: builtins.int = ...,
        status: global___Status | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["status", b"status"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "attributes",
            b"attributes",
            "dropped_attributes_count",
            b"dropped_attributes_count",
            "dropped_events_count",
            b"dropped_events_count",
            "dropped_links_count",
            b"dropped_links_count",
            "end_time_unix_nano",
            b"end_time_unix_nano",
            "events",
            b"events",
            "flags",
            b"flags",
            "kind",
            b"kind",
            "links",
            b"links",
            "name",
            b"name",
            "parent_span_id",
            b"parent_span_id",
            "span_id",
            b"span_id",
            "start_time_unix_nano",
            b"start_time_unix_nano",
            "status",
            b"status",
            "trace_id",
            b"trace_id",
            "trace_state",
            b"trace_state",
        ],
    ) -> None: ...

global___Span = Span

@typing_extensions.final
class Status(google.protobuf.message.Message):
    """The Status type defines a logical error model that is suitable for different
    programming environments, including REST APIs and RPC APIs.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _StatusCode:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _StatusCodeEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[
            Status._StatusCode.ValueType
        ],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        STATUS_CODE_UNSET: Status._StatusCode.ValueType  # 0
        """The default status."""
        STATUS_CODE_OK: Status._StatusCode.ValueType  # 1
        """The Span has been validated by an Application developer or Operator to 
        have completed successfully.
        """
        STATUS_CODE_ERROR: Status._StatusCode.ValueType  # 2
        """The Span contains an error."""

    class StatusCode(_StatusCode, metaclass=_StatusCodeEnumTypeWrapper):
        """For the semantics of status codes see
        https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/trace/api.md#set-status
        """

    STATUS_CODE_UNSET: Status.StatusCode.ValueType  # 0
    """The default status."""
    STATUS_CODE_OK: Status.StatusCode.ValueType  # 1
    """The Span has been validated by an Application developer or Operator to 
    have completed successfully.
    """
    STATUS_CODE_ERROR: Status.StatusCode.ValueType  # 2
    """The Span contains an error."""

    MESSAGE_FIELD_NUMBER: builtins.int
    CODE_FIELD_NUMBER: builtins.int
    message: builtins.str
    """A developer-facing human readable error message."""
    code: global___Status.StatusCode.ValueType
    """The status code."""
    def __init__(
        self,
        *,
        message: builtins.str = ...,
        code: global___Status.StatusCode.ValueType = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "code", b"code", "message", b"message"
        ],
    ) -> None: ...

global___Status = Status



from typing import (
    Any,
    List,
)

from boss.model import User
from boss.model.repository import UserRepository
from .._BaseInteractor import BaseInteractor


class GetUsersListInteractor(BaseInteractor):

    def __init__(
        self,
        user_repository: UserRepository,
    ) -> None:

        self.user_repository = user_repository

    def execute(
        self,
        request: Any,
    ) -> List[User]:

        return self.user_repository.get_users_list()



# postnummer.py - functions for handling Swedish postal codes
#
# Copyright (C) 2021 Michele Ciccozzi
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""Postcode (the Swedish postal code).

The Swedish postal code consists of three numbers followed by two numbers,
separated by a single space.

More information:

* https://en.wikipedia.org/wiki/Postal_codes_in_Sweden
* https://sv.wikipedia.org/wiki/Postnummer_i_Sverige

>>> validate('114 18')
'11418'
>>> validate('SE-11418')
'11418'
>>> validate('1145 18')
Traceback (most recent call last):
    ...
InvalidLength: ...
>>> format('11418')
'114 18'
"""

from stdnum.exceptions import *
from stdnum.util import clean, isdigits


def compact(number):
    """Convert the number to the minimal representation. This strips the
    number of any valid separators and removes surrounding whitespace."""
    number = clean(number, ' -').upper().strip()
    if number.startswith('SE'):
        number = number[2:]
    return number


def validate(number):
    """Check if the number is in the correct format. This currently does not
    check whether the code corresponds to a real address."""
    number = compact(number)
    if not isdigits(number) or number.startswith('0'):
        raise InvalidFormat()
    if len(number) != 5:
        raise InvalidLength()
    return number


def is_valid(number):
    """Check if the number is a valid postal code."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False


def format(number):
    """Reformat the number to the standard presentation format."""
    number = compact(number)
    return '%s %s' % (number[:3], number[3:])



from create_table_order import create_table_order
from deselect_table import (
    check_tab_user_sector,
    deselect_table,
)
from do_change_seat_number import do_change_seat_number
from do_reopen_table import reopen_table
from do_split_items import do_split_items
from do_split_items_equally import do_split_items_equally
from store_table_service import store_table_service
from table_service_actions import validate_if_table_opened_another_pos
from create_tab import get_tab_info
from do_reopen_table import reopen_table
from deselect_table import (
    check_tab_user_sector,
    deselect_table,
)



# -*- coding: utf-8 -*-

import base64
import bisect
import glob
import json
import logging
import os
import os.path
import re
import shutil
import zipfile
import zlib
from datetime import datetime, timedelta
from decimal import Decimal as D
from xml.etree import cElementTree as eTree

from helper import remove_accents
from modelcommon.fiscalwrapper import FiscalizationException

from bustoken import (
    TK_FISCALWRAPPER_CANCEL_ORDER,
    TK_FISCALWRAPPER_CHANGE_CONTINGENCY_STATUS,
    TK_FISCALWRAPPER_EXPORT_FISCAL_FILES,
    TK_FISCALWRAPPER_GET_BASE_REDUCTION,
    TK_FISCALWRAPPER_GET_CERTIFICATE_EXPIRATION_DAYS,
    TK_FISCALWRAPPER_GET_CONTINGENCY_STATUS,
    TK_FISCALWRAPPER_GET_CST,
    TK_FISCALWRAPPER_GET_FISCAL_XML,
    TK_FISCALWRAPPER_GET_IBPT_TAX,
    TK_FISCALWRAPPER_GET_LAST_ORDER_ID,
    TK_FISCALWRAPPER_GET_NF_TYPE,
    TK_FISCALWRAPPER_GET_ORDERS_DATA,
    TK_FISCALWRAPPER_GET_TIP,
    TK_FISCALWRAPPER_PROCESS_REQUEST,
    TK_FISCALWRAPPER_RE_SIGN_XML,
    TK_FISCALWRAPPER_SAT_OPERATIONAL_STATUS_REQUEST,
    TK_FISCALWRAPPER_SAT_PAYMENT_STATUS,
    TK_FISCALWRAPPER_SAT_PROCESS_PAYMENT,
    TK_FISCALWRAPPER_SAT_STORE_STATUS,
    TK_FISCALWRAPPER_SEARCH_REQUEST,
    TK_FISCALWRAPPER_SEFAZ_CONNECTIVITY,
    TK_FISCALWRAPPER_SITUATION,
    TK_SAT_OPERATIONAL_STATUS_REQUEST,
    TK_SAT_PROCESS_PAYMENT,
    TK_SAT_PAYMENT_STATUS,
    TK_SAT_STATUS_REQUEST,
    TK_FISCALWRAPPER_CAN_FISCALIZE_ORDER,
)
from typing import (
    List,
    Optional,
    Tuple,
    Union,
)

from common import FiscalParameterController
from comp_exceptions import (
    EmptySearchException,
    VoidOrderException,
    StoreNotOperatorWithSatException,
    OrderWaitingProcessingException,
    WsVersionNotFound,
    MaxSaleItemsQtyException,
)
from fiscalinterface import FiscalProcessor
from fiscalpaymentparser import convert_order_tenders_to_fiscal_tenders
from fiscalpersistence import FiscalDataRepository
from old_helper import OrderTaker, round_half_away_from_zero, remove_xml_namespace, \
    convert_from_utf_to_localtime, regex_search
from ibpttaxprocessor import IbptTaxProcessor
from messagehandler import EventHandler
from msgbus import (
    TK_SYS_NAK,
    TK_SYS_ACK,
    MBEasyContext,
    MBException,
    FM_PARAM,
    MBMessage,
)
from nfce import (
    NfceRequestBuilder,
    NfceConnectivityTester,
    NfceContingencia,
    NfceProcessor,
)
from nfcecanceler import NfceCanceler
from nfcedisabler import OrderDisabler
from order_retriever import OrderRetrieverService
from persistence import Driver as DBDriver
from sat import SatProcessor, SatServiceFinder
from services import (
    SequencerService,
    OrderService,
)
from sysactions import (
    get_model,
    get_posot,
    translate_message,
)
from systools import sys_log_exception

from models import NfceCstatCode

logger = logging.getLogger("FiscalWrapper")
logger_coupons = logging.getLogger("FiscalWrapperCoupons")
logger_canceler = logging.getLogger("FiscalWrapperNfceCanceler")


class FiscalWrapperEventHandler(EventHandler):

    def __init__(
        self,                                   # type: FiscalWrapperEventHandler
        mbcontext,                              # type: MBEasyContext
        fiscal_processor,                       # type: Union[FiscalProcessor, NfceProcessor, SatProcessor]
        nfce_connectivity_tester,               # type: NfceConnectivityTester
        order_disabler,                         # type: OrderDisabler
        nfce_canceler,                          # type: NfceCanceler
        fiscal_parameter_controller,            # type: FiscalParameterController
        xml_enviados,                           # type: str
        xml_erros,                              # type: str
        export_xml_path,                        # type: str
        fiscal_sent_dir,                        # type: str
        nf_type,                                # type: str
        versao_ws,                              # type: str
        ibpt_tax_processor,                     # type: IbptTaxProcessor
        order_retriever_service,                # type: OrderRetrieverService
        xml_inconsistentes,                     # type: str
        nfce_contingencia,                      # type: NfceContingencia
        period_to_retry_orders_with_exception,  # type: int
        synchronous_mode,                       # type: bool
        sequencer_service,                      # type: SequencerService
    ):
        # type: (...) -> None
        super(FiscalWrapperEventHandler, self).__init__(mbcontext)

        self.fiscal_processor = fiscal_processor
        self.nf_type = nf_type  # type: str
        self.nfce_connectivity_tester = nfce_connectivity_tester
        self.order_disabler = order_disabler
        self.order_retriever_service = order_retriever_service
        self.xml_enviados = xml_enviados
        self.xml_erros = xml_erros
        self.export_xml_path = export_xml_path
        self.fiscal_sent_dir = fiscal_sent_dir
        self.nfce_canceler = nfce_canceler
        self.versao_ws = versao_ws
        self.fiscal_parameter_controller = fiscal_parameter_controller
        self.ibpt_tax_processor = ibpt_tax_processor
        self.xml_inconsistentes = xml_inconsistentes
        self.nfce_contingencia = nfce_contingencia
        self.period_to_retry_orders_with_exception = period_to_retry_orders_with_exception
        self.synchronous_mode = synchronous_mode
        self.sequencer_service = sequencer_service
        self.order_service = OrderService(mb_context=mbcontext)

    def get_handled_tokens(self):
        return [
            TK_FISCALWRAPPER_CANCEL_ORDER,
            TK_FISCALWRAPPER_CHANGE_CONTINGENCY_STATUS,
            TK_FISCALWRAPPER_EXPORT_FISCAL_FILES,
            TK_FISCALWRAPPER_GET_BASE_REDUCTION,
            TK_FISCALWRAPPER_GET_CERTIFICATE_EXPIRATION_DAYS,
            TK_FISCALWRAPPER_GET_CONTINGENCY_STATUS,
            TK_FISCALWRAPPER_GET_CST,
            TK_FISCALWRAPPER_GET_FISCAL_XML,
            TK_FISCALWRAPPER_GET_IBPT_TAX,
            TK_FISCALWRAPPER_GET_LAST_ORDER_ID,
            TK_FISCALWRAPPER_GET_NF_TYPE,
            TK_FISCALWRAPPER_GET_ORDERS_DATA,
            TK_FISCALWRAPPER_GET_TIP,
            TK_FISCALWRAPPER_PROCESS_REQUEST,
            TK_FISCALWRAPPER_RE_SIGN_XML,
            TK_FISCALWRAPPER_SAT_OPERATIONAL_STATUS_REQUEST,
            TK_FISCALWRAPPER_SAT_PAYMENT_STATUS,
            TK_FISCALWRAPPER_SAT_PROCESS_PAYMENT,
            TK_FISCALWRAPPER_SAT_STORE_STATUS,
            TK_FISCALWRAPPER_SEARCH_REQUEST,
            TK_FISCALWRAPPER_SEFAZ_CONNECTIVITY,
            TK_FISCALWRAPPER_SITUATION,
            TK_SAT_OPERATIONAL_STATUS_REQUEST,
            TK_SAT_PAYMENT_STATUS,
            TK_SAT_PROCESS_PAYMENT,
            TK_FISCALWRAPPER_CAN_FISCALIZE_ORDER,
        ]

    def handle_message(self, msg):
        # type: (MBMessage) -> Union[None, bool]
        try:
            if msg.token == TK_FISCALWRAPPER_PROCESS_REQUEST:
                response = self._handle_process_request(msg)
            elif msg.token == TK_FISCALWRAPPER_GET_CERTIFICATE_EXPIRATION_DAYS:
                response = self._handle_get_certificate_expiration()
            elif msg.token == TK_FISCALWRAPPER_SEARCH_REQUEST:
                response = self._handle_search_request()
            elif msg.token == TK_FISCALWRAPPER_SEFAZ_CONNECTIVITY:
                response = self._handle_sefaz_connectivity_request()
            elif msg.token == TK_FISCALWRAPPER_SAT_OPERATIONAL_STATUS_REQUEST:
                response = self._handle_sat_op_status_request(msg)
            elif msg.token == TK_FISCALWRAPPER_EXPORT_FISCAL_FILES:
                response = self._export_fiscal_files(msg)
            elif msg.token == TK_FISCALWRAPPER_GET_NF_TYPE:
                response = self._handle_get_nftype(msg)
            elif msg.token == TK_FISCALWRAPPER_SAT_PROCESS_PAYMENT:
                response = self._handle_sat_process_payment_request(msg)
            elif msg.token == TK_FISCALWRAPPER_SAT_PAYMENT_STATUS:
                response = self._handle_sat_payment_status_request(msg)
            elif msg.token == TK_FISCALWRAPPER_CANCEL_ORDER:
                response = self._handle_cancel_order(msg)
            elif msg.token == TK_FISCALWRAPPER_RE_SIGN_XML:
                response = self._handle_re_sign_xml()
            elif msg.token == TK_FISCALWRAPPER_SITUATION:
                response = self._handle_sefaz_situation(msg)
            elif msg.token == TK_FISCALWRAPPER_GET_FISCAL_XML:
                response = self._handle_fiscal_xml_request(msg.data, True)
            elif msg.token == TK_FISCALWRAPPER_GET_CST:
                response = self._handle_get_csts(msg)
            elif msg.token == TK_FISCALWRAPPER_GET_BASE_REDUCTION:
                response = self._handle_get_base_reductions(msg)
            elif msg.token == TK_FISCALWRAPPER_GET_TIP:
                response = self._handle_get_tip_params(msg)
            elif msg.token == TK_FISCALWRAPPER_GET_IBPT_TAX:
                response = self._handle_get_ibpt_tax(msg)
            elif msg.token == TK_FISCALWRAPPER_GET_CONTINGENCY_STATUS:
                response = self._handle_get_contingency_status(msg)
            elif msg.token == TK_FISCALWRAPPER_CHANGE_CONTINGENCY_STATUS:
                response = self._handle_change_contingency_status(msg)
            elif msg.token == TK_FISCALWRAPPER_SAT_STORE_STATUS:
                response = self._handle_sat_status_request()
            elif msg.token == TK_FISCALWRAPPER_GET_ORDERS_DATA:
                try:
                    orders_data_list_str = self.order_retriever_service.get_orders_data(msg.data)

                    msg.token = TK_SYS_ACK
                    self.mbcontext.MB_ReplyMessage(msg, data=orders_data_list_str)
                    return
                except Exception as ex:
                    logger.exception("Error handling TK_FISCALWRAPPER_GET_ORDERS_DATA")
                    msg.token = TK_SYS_NAK
                    self.mbcontext.MB_ReplyMessage(msg, data=ex.message)
                    return
            elif msg.token == TK_FISCALWRAPPER_GET_LAST_ORDER_ID:
                try:
                    last_order_id = self.order_retriever_service.get_last_order_id()

                    msg.token = TK_SYS_ACK
                    self.mbcontext.MB_ReplyMessage(msg, data=last_order_id)
                    return
                except Exception as ex:
                    logger.exception("Error handling TK_FISCALWRAPPER_GET_LAST_ORDER_ID")
                    msg.token = TK_SYS_NAK
                    self.mbcontext.MB_ReplyMessage(msg, data=ex.message)
                    return
            elif msg.token == TK_FISCALWRAPPER_CAN_FISCALIZE_ORDER:
                try:
                    total_gross, tip_amount = msg.data.split("\0")
                    self._can_fiscalize_order(
                        total_gross=float(total_gross or "0.0"),
                        tip_amount=float(tip_amount or "0.0"),
                    )
                    msg.token = TK_SYS_ACK
                    self.mbcontext.MB_ReplyMessage(msg, data='')
                    return
                except FiscalizationException as ex:
                    msg.token = TK_SYS_NAK
                    error = json.dumps({
                        "message": ex.message,
                        "error_code": ex.error_code
                    })
                    self.mbcontext.MB_ReplyMessage(msg, data=error)
                return
            else:
                response = (False, "")

            self._log_response_if_error(response=response)

            msg.token = TK_SYS_ACK if response[0] else TK_SYS_NAK
            self.mbcontext.MB_ReplyMessage(msg, data=response[1])

        except StoreNotOperatorWithSatException as ex:
            msg.token = TK_SYS_NAK
            self.mbcontext.MB_ReplyMessage(msg, data=ex.message)
        except (Exception,) as ex:
            sys_log_exception("Unexpected exception when handling event {0}. Error message: {1}".format(
                msg.token,
                ex,
            ))
            msg.token = TK_SYS_NAK
            self.mbcontext.MB_ReplyMessage(msg, data=str(ex))

        return False

    @staticmethod
    def _log_response_if_error(
        response,   # type: Tuple[bool, str]
    ):
        # type: (...) -> None

        if len(response) == 2:
            response_status = response[0]
            response_message = response[1]

            if response_status is False:
                logger.error(response_message)

    def terminate_event(self):
        self.fiscal_processor.terminate()

    def _handle_get_nftype(self, _):
        # type: (MBMessage) -> (str, str)
        return True, self.nf_type

    def _handle_get_certificate_expiration(self):
        if self.nf_type == "NFCE":
            ret = self.fiscal_processor.do_validation(get_days_to_expiration=True)
            if not ret[0]:
                return False, ret[1]
            return True, ret[1]

    def _handle_process_request(
        self,           # type: FiscalWrapperEventHandler
        msg,            # type: MBMessage
    ):
        # type: (...) -> Tuple[bool, str]

        ret = self.fiscal_processor.do_validation()
        if not ret[0]:
            return False, '\0'.join(map(str, (False, str(ret[1]))))

        data = msg.data.split('\0')
        posid = data[0]
        orderid = data[1] if len(data) > 1 else ""
        model = get_model(posid)
        pos_ot = get_posot(model)
        optional_fields = {
            "fields_to_show": {
                "SaleLine": ["customParams"]
            },
        }
        order = eTree.XML(
            pos_ot.orderPicture(
                posid=posid,
                orderid=orderid,
                optional_fields=optional_fields,
            )
        )
        order = order.find("Order") if order.find("Order") else order

        if self._order_is_already_fiscalized(order=order):
            return True, ""

        not_allowed_max_value = self.fiscal_parameter_controller.validate_fiscal_max_value(
            model=model,
            order=order,
        )

        if not_allowed_max_value:
            return False, ""

        with FiscalDataRepository(self.mbcontext) as fiscal_repository:  # type: FiscalDataRepository
            fiscal_ok = False
            try:
                fiscal_id_property = "FISCAL_ID"
                fiscal_id_node = order.find("CustomOrderProperties/OrderProperty[@key='{}']".format(fiscal_id_property))

                if fiscal_id_node is not None:
                    fiscal_id_node = fiscal_id_node.get("value")

                if not fiscal_id_node:
                    fiscal_id_node = self._set_fiscal_id_on_custom_property(pos_id=posid)

                tenders = convert_order_tenders_to_fiscal_tenders(order)

                fiscal_data = self.fiscal_processor.request_fiscal(
                    posid=posid,
                    order=order,
                    tenders=tenders,
                    paf=self.nf_type == "PAF",
                    fiscal_id=str(fiscal_id_node),
                )
                fiscal_ok = True

                if self.nf_type in ("SAT", "MFE"):
                    sat_fiscal_processor = self.fiscal_processor  # type: SatProcessor
                    sat_used = sat_fiscal_processor.last_sat_used[posid]
                    sat_used = sat_used[3:]
                    response_xml = eTree.tostring(fiscal_data.find("FiscalData"))
                    base64_response = base64.b64encode(response_xml)
                    fiscal_repository.start_save_fiscal_data_thread(
                        pos_id=posid,
                        order=order,
                        xml_request=fiscal_data.find("XmlRequest").text.strip(),
                        numero_sat=sat_used,
                        sent_to_nfce=1,
                        xml_response=base64_response,
                    )
                else:
                    request_xml = eTree.XML(fiscal_data[0])
                    xml_response_base64 = None
                    if fiscal_data[1] is not None:
                        xml_response_base64 = base64.b64encode(fiscal_data[1])
                    idx_start = fiscal_data[0].find("<NFe")
                    idx_final = int(str(fiscal_data[0].find("</NFe"))) + 6
                    xml_string = fiscal_data[0][idx_start:idx_final]

                    authorization_number = NfceRequestBuilder.NAMESPACE_AUTORIZACAO_4
                    if self.versao_ws in (1, 3):
                        authorization_number = NfceRequestBuilder.NAMESPACE_AUTORIZACAO

                    base_key = "{{{0}}}Body/{{{1}}}nfeDadosMsg/{{{2}}}enviNFe/{{{2}}}NFe/{{{2}}}infNFe/"
                    base_key += "{{{2}}}ide/{{{2}}}tpEmis"
                    key_to_find = base_key.format(
                        NfceRequestBuilder.NAMESPACE_SOAP,
                        authorization_number,
                        NfceRequestBuilder.NAMESPACE_NFE,
                    )

                    contingencia = request_xml.find(key_to_find).text
                    if contingencia != "9":
                        xml_string = fiscal_data[2]
                    xml_base64 = base64.b64encode(xml_string)

                    fiscal_repository.start_save_fiscal_data_thread(
                        pos_id=posid,
                        order=order,
                        xml_request=xml_base64,
                        numero_sat="00",
                        sent_to_nfce=0 if contingencia == "9" else 1,
                        xml_response=xml_response_base64,
                    )

                return True, ""

            except VoidOrderException as ex:
                fiscal_ok = False
                return False, '\0'.join(map(str, (fiscal_ok, str(ex))))

            except MBException as ex:
                logger.exception("MessageBus Exception")
                return False, '\0'.join(map(str, (fiscal_ok, ex.message)))

            except OrderWaitingProcessingException as ex:
                fiscal_ok = False
                return False, "\0".join(map(str, (fiscal_ok, str(ex))))

            except MaxSaleItemsQtyException as ex:
                logger.exception("Quantidade máxima de itens no pedido atingida")
                return False, '\0'.join(map(str, (fiscal_ok, ex.message)))

            except Exception as ex:
                logger.exception("Nota Rejeitada pela SEFAZ")
                return False, '\0'.join(map(str, (fiscal_ok, str(ex))))

    def _set_fiscal_id_on_custom_property(
        self,       # type: FiscalWrapperEventHandler
        pos_id,     # type: str
    ):
        # type: (...) -> int

        current_fiscal_id = self.sequencer_service.get_current_fiscal_id()
        try:
            fiscal_id = self.sequencer_service.build_next_fiscal_id()
            self.order_service.set_custom_property(
                pos_id=pos_id,
                key="FISCAL_ID",
                value=str(fiscal_id),
                blk_notify=True,
            )
            return fiscal_id
        except (Exception, ):
            self.sequencer_service.set_fiscal_id(fiscal_id=current_fiscal_id)
            raise

    def _order_is_already_fiscalized(
        self,   # type: FiscalWrapperEventHandler
        order,  # type: eTree.Element
    ):
        # type: (...) -> bool

        pos_id = order.get("posId")
        order_id = order.get("orderId")

        fiscalization_date = order.find("CustomOrderProperties/OrderProperty[@key='FISCALIZATION_DATE']")
        if fiscalization_date is not None:
            logger.info(
                "Found Fiscalization Date for OrderId: {}, PosId: {}, FiscalizationDate: {}".format(
                    order_id,
                    pos_id,
                    fiscalization_date.get("value"),
                )
            )

            fiscal_xml = order.find("CustomOrderProperties/OrderProperty[@key='FISCAL_XML']")
            if fiscal_xml is not None:
                fiscal_xml = fiscal_xml.get("value", "")
                logger.info("Found Fiscal XML starts with: {}...".format(fiscal_xml[:15]))
                return True

            response = self._handle_fiscal_xml_request(
                data=order_id,
                return_data=True,
            )
            has_fiscal_xml = response[0]
            if has_fiscal_xml:
                fiscal_xml = response[1]
                with FiscalDataRepository(self.mbcontext) as fiscal_repository:
                    fiscal_repository.set_fiscal_xml_custom_property(
                        pos_id=pos_id,
                        xml_request=fiscal_xml,
                        order_id=order_id,
                    )
                return True

    def _handle_sefaz_connectivity_request(self):
        if self.nf_type == "NFCE":
            self.nfce_connectivity_tester.logger = logger
            if self.nfce_connectivity_tester.test_connectivity():
                return True, "Conexao com Sefaz OK!"
            else:
                return False, "Nao foi possivel se conectar com a Sefaz."
        else:
            return False, "Loja nao opera com NFCE"

    def _handle_sefaz_situation(
        self,   # type: FiscalWrapperEventHandler
        msg,    # type: MBMessage
    ):
        # type: (...) -> Tuple[bool, str]

        if self.nf_type == "NFCE":
            initial_date, final_date = msg.data.split('\0')
            return self.fiscal_processor.check_nf_situation(
                initial_date=initial_date,
                final_date=final_date,
            )
        else:
            return False, "Loja nao opera com NFCE"

    def _handle_get_csts(self, _):
        return True, self.fiscal_parameter_controller.get_all_products_csts()

    def _handle_get_base_reductions(self, _):
        return True, self.fiscal_parameter_controller.get_all_products_base_reductions()

    def _handle_get_tip_params(self, _):
        return True, self.fiscal_parameter_controller.get_tip_params()

    def _handle_get_ibpt_tax(self, msg):
        order_xml = eTree.XML(msg.data)
        taxes_dict = self.ibpt_tax_processor.ibpt_tax_calculator(order_xml)
        return True, str(taxes_dict)

    def _handle_get_contingency_status(self, _):
        return True, "Enabled" if self.nfce_contingencia.contingencia else "Disabled"

    def _handle_change_contingency_status(self, _):
        self.nfce_contingencia.force_contingency = not self.nfce_contingencia.force_contingency
        if self.nfce_contingencia.force_contingency:
            logger.info("Entrando em contingência forçada via menu suporte")
            self.nfce_contingencia.entra_contingencia("Contingencia forcada")
        else:
            self.nfce_contingencia.contingencia = False
            self.nfce_contingencia.process_now = True
            self.nfce_contingencia.start_contingencia()

        return True, "Contingency status successfully changed"

    def handle_event(self, subject, evt_type, data, msg):
        if subject == "ORDER_MODIFIED" and evt_type == "PAID":
            order = eTree.XML(data).find("Order")
            order_id = order.get("orderId")
            with FiscalDataRepository(self.mbcontext) as fiscal_repository:  # type: FiscalDataRepository
                fiscal_repository.set_order_picture(order_id, base64.b64encode(eTree.tostring(order)))
        elif subject == "ORDER_MODIFIED" and evt_type == "VOID_ORDER":
            order = eTree.XML(data).find(".//Order")
            order_id = order.get("orderId")
            with FiscalDataRepository(self.mbcontext) as fiscal_repository:  # type: FiscalDataRepository
                fiscal_repository.set_order_canceled(order_id)
        elif subject == "DisableNfceOrder":
            self._handle_disable_orders()
        elif subject == "ReSignXMLs":
            self._handle_re_sign_xml()
        elif subject == "OrderManagerFiscalXmlRequest":
            self._handle_fiscal_xml_request(data)
        elif subject == "CheckIfFiscalXmlIsCreatedInFolder":
            self._handle_check_if_fiscal_xml_is_created_in_folder()
        elif subject == "RetryFiscalOrdersWithException":
            self._handle_retry_fiscal_orders_with_exception()

    def _handle_search_request(self):
        self._check_store_operator_with_sat()

        sat_status_list = {}
        sat_service_finder = self.fiscal_processor.sat_service_finder  # type: SatServiceFinder

        pos_sat_list = sat_service_finder.pos_sat_list
        sat_list_to_search = pos_sat_list if pos_sat_list else range(0, sat_service_finder.max_sat_number + 1)

        for i in sat_list_to_search:
            sat_service_name = "SAT%s" % str(i).zfill(2)
            try:
                ret = self.mbcontext.MB_EasySendMessage(
                    dest_name=sat_service_name,
                    token=TK_SAT_STATUS_REQUEST,
                    format=FM_PARAM,
                    timeout=100 * 100000,
                )

                if ret.token == TK_SYS_ACK:
                    sat_status_list[sat_service_name] = ret.data
                else:
                    sat_status_list[sat_service_name] = "ERROR"
            except Exception as ex:
                logger.error("Erro buscando SAT - Exception: %s", str(ex))
                sat_status_list[sat_service_name] = "Desabilitado"

        return True, str(sat_status_list)

    def _handle_disable_orders(self):
        if not self.nf_type == "NFCE":
            return
        try:
            self.order_disabler.disable_all_orders()
        except (Exception,):
            logger.exception("Erro inutilizando notas")

    def _handle_sat_status_request(
        self,
    ):
        # type: (...) -> Tuple[bool, str]

        self._check_store_operator_with_sat()

        _, search_request = self._handle_search_request()
        search_request = eval(search_request)
        for key, value in search_request.iteritems():
            if str(value).upper() != "OK":
                continue

            logger.info("Active SAT {}".format(str(key)))
            return True, str(key)

        return False, "Sem SAT ativo na loja"

    def _handle_sat_op_status_request(
        self,
        msg,                    # type: MBMessage
    ):
        # type: (...) -> Tuple[bool, str]

        pos_id = msg.data

        self._check_store_operator_with_sat()

        sat_service_finder = self.fiscal_processor.sat_service_finder  # type: SatServiceFinder
        pos_sat_list = sat_service_finder.pos_sat_list

        pos_id_list = [pos_id] if not pos_sat_list or str(pos_id) in pos_sat_list else []

        sat_info_list = []

        for current_pos_id in pos_id_list:
            sat_service_name = "SAT{}".format(current_pos_id.zfill(2))
            try:
                response = self.mbcontext.MB_EasySendMessage(
                    dest_name=sat_service_name,
                    token=TK_SAT_OPERATIONAL_STATUS_REQUEST,
                    format=FM_PARAM,
                    timeout=100 * 100000,
                )
                sat_info_list.append(response.data)
            except MBException as ex:
                logger.info("Error searching SAT on POS#{}: {}".format(pos_id, str(ex)))
                sat_info_list.append("ERROR - Desabilitado")

        if not pos_id_list:
            sat_info_list.append("ERROR - Desabilitado")

        return True, str(sat_info_list)

    def _export_fiscal_files(self, msg):
        nok = False, 'Erro ao exportar arquivos fiscais'

        try:
            parse_ok, start_date, end_date = self._parse_date_from_fiscal_screen(msg.data)
            if not parse_ok:
                return nok

            try:
                folder_ok, folders = self._find_fiscal_folders(start_date, end_date, msg.data)
                if not folder_ok:
                    return nok
            except EmptySearchException as ex:
                return False, ex.message

            zip_ok, zip_path = self.zip_folders(folders)
            if not zip_ok:
                return nok

            return True, zip_path

        except Exception as ex:
            logger.error("Erro ao exportar arquivos fiscais %s", str(ex))

        return nok

    @staticmethod
    def _parse_date_from_fiscal_screen(dates):
        try:
            # parse das datas: yyyy/mm/dd;yyyy/mm/dd
            match = re.search(r'(\d+/\d+/\d+);(\d+/\d+/\d+)', dates)
            if match:
                dateSplit = dates.split(";")
                startDate = dateSplit[0]
                endDate = dateSplit[1]
                return True, startDate, endDate

        except Exception as ex:
            logger.error("Erro ao parsear datas (_parse_date_from_fiscal_screen) %s", str(ex))

        return False, '', ''

    def _find_fiscal_folders(self, start_date, end_date, msg):
        try:
            logger.info("start_date: " + start_date)
            logger.info("end_date: " + end_date)

            xml_type = self.xml_enviados if msg.split(";")[2] == '1' else self.xml_erros
            path = os.path.join(self.fiscal_sent_dir, xml_type)
            logger.info("path: " + path)

            export_folder = os.path.join(path, 'ExportTemp')
            logger.info("export_folder: " + export_folder)

            folders = filter(lambda f: os.path.isdir(f), glob.glob(path + '/*/*/*'))
            logger.info("lambda: " + str((lambda f: os.path.isdir(f), glob.glob(path + '/*/*/*'))))

            folders = [s.replace(path + "\\", '').replace(path + "/", '').replace('\\', '/') for s in folders]
            logger.info("folders after replace: " + str(folders))

            folders.sort()
            logger.info("folders after sort: " + str(folders))

            folders = folders[bisect.bisect_left(folders, start_date):bisect.bisect_right(folders, end_date)]
            logger.info("folders bisect_left: " + str(bisect.bisect_left(folders, start_date)))
            logger.info("folders bisect_right: " + str(bisect.bisect_right(folders, end_date)))
            logger.info("folders after bisect: " + str(folders))

            if folders:
                shutil.rmtree(export_folder, True)
                for item in folders:
                    shutil.copytree(os.path.join(path, item), os.path.join(export_folder, item))

                return True, export_folder
            else:
                raise EmptySearchException("Nenhum arquivo encontrado")

        except EmptySearchException as ex:
            raise ex

        except Exception as ex:
            logger.error("Erro ao buscar diretorios (_find_fiscal_folders) %s", str(ex))

        return False, ''

    def zip_folders(self, folder):
        try:
            zip_name = 'export'
            zip_complete_name = zip_name + '.zip'

            try:
                os.remove(os.path.join(self.export_xml_path, zip_complete_name))
            except OSError:
                pass

            self.zip(folder, zip_name)
            shutil.move(zip_complete_name, self.export_xml_path)
            shutil.rmtree(folder, True)
            return True, zip_complete_name
        except Exception as ex:
            logger.error("Erro ao zipar o diretorio temporario (zip_folders) %s", str(ex))
            shutil.rmtree(folder, True)

        return False, ''

    @staticmethod
    def zip(src, dst):
        zf = zipfile.ZipFile("%s.zip" % dst, "w", zipfile.ZIP_DEFLATED)
        abs_src = os.path.abspath(src)
        for dirname, subdirs, files in os.walk(src):
            for filename in files:
                absname = os.path.abspath(os.path.join(dirname, filename))
                arcname = absname[len(abs_src) + 1:]
                zf.write(absname, arcname)
        zf.close()

    def _handle_sat_process_payment_request(self, msg):
        if self.nf_type not in ("MFE",):
            return True, ''
        sat_service_name = self.fiscal_processor.sat_service_finder.find_and_lock_sat_service(msg.data.split("\0")[1])
        msg = self.mbcontext.MB_EasySendMessage(
            dest_name=sat_service_name,
            token=TK_SAT_PROCESS_PAYMENT,
            format=FM_PARAM,
            data=msg.data,
        )
        self.fiscal_processor.sat_service_finder.unlock_sat_service(sat_service_name)
        return True, msg.data

    def _handle_sat_payment_status_request(self, msg):
        if self.nf_type not in ("MFE",):
            return True, ''

        sat_service_name = self.fiscal_processor.sat_service_finder.find_and_lock_sat_service(msg.data.split("\0")[1])
        msg = self.mbcontext.MB_EasySendMessage(sat_service_name, TK_SAT_PAYMENT_STATUS, format=FM_PARAM, data=msg.data)
        self.fiscal_processor.sat_service_finder.unlock_sat_service(sat_service_name)
        return True, msg.data

    def _handle_cancel_order(
        self,   # type: FiscalWrapperEventHandler
        msg,    # type: MBMessage
    ):
        # type: (...) -> Tuple[bool, str]

        received_data = msg.data.split("|")
        pos_id = received_data[0]
        order_id = received_data[1]
        model = get_model(posid=pos_id)
        pos_ot = get_posot(model=model)

        model = get_model(pos_id)
        pos_ot = get_posot(model)

        optional_fields = {
            "fields_to_show": {
                "SaleLine": ["customParams"]
            },
        }
        order = eTree.XML(
            pos_ot.orderPicture(
                posid=pos_id,
                orderid=order_id,
                optional_fields=optional_fields,
            )
        )
        if order.tag == "Orders":
            order = order.find("Order")

        order_id = order.get('orderId')
        pos_id = order.get('posId')

        justification = self._get_cancel_order_justification(
            model=model,
            received_data=received_data,
        )
        log_message = "Processando cancelamento do order: {}. Justificativa: {}"
        logger_canceler.info(log_message.format(order_id, justification))

        fiscalization_date = order.find(".//CustomOrderProperties/OrderProperty[@key='FISCALIZATION_DATE']")
        if fiscalization_date is None:
            return False, "Pedido ainda não fiscalizado"

        formatted_fiscalization_date = datetime.strptime(fiscalization_date.get("value"), "%Y-%m-%dT%H:%M:%S")
        since_datetime = datetime.now() - timedelta(minutes=30)
        if convert_from_utf_to_localtime(formatted_fiscalization_date).replace(tzinfo=None) < since_datetime:
            return False, "Pedido excedeu o tempo limite para cancelamento fiscal (30 minutos)"

        response = (False, msg.data)
        try:
            if self.nf_type == 'NFCE':
                response = self._cancel_nfce(
                    pos_id=pos_id,
                    order_id=order_id,
                    justification=justification,
                )

            elif self.nf_type in ['SAT', 'MFE']:
                response = self._cancel_sat(
                    pos_id=pos_id,
                    order_id=order_id,
                )

            return response
        finally:
            if response and len(response) == 2 and response[0] is True:
                logger_canceler.info("Cancelamento realizado com sucesso. Status: {}".format(response[1]))

    @staticmethod
    def _get_cancel_order_justification(
        model,          # type: eTree.Element
        received_data,  # type: List[str]
    ):
        # type: (...) -> str

        justification = received_data[2] if len(received_data) > 2 else None
        justification = justification if justification is not None else "USER_FISCAL_CANCELLATION"
        justification = remove_accents(
            text=translate_message(
                model=model,
                message_key=justification,
            ),
        )
        justification = "CANCELAMENTO: {}".format(justification)

        return justification

    def _cancel_nfce(
        self,
        pos_id,         # type: str
        order_id,       # type: str
        justification,  # type: str
    ):
        # type: (...) -> Tuple[bool, str]

        with FiscalDataRepository(mbcontext=self.mbcontext) as fiscal_repository:
            order_data = fiscal_repository.get_xml_response(order_id=order_id)
            if order_data is None:
                return False, "Só é possível cancelar vendas já autorizadas"

            response = self.nfce_canceler.cancel_order(
                order_response=order_data,
                justification=justification,
                order_id=order_id,
            )

            if response.get("status"):
                self.order_service.set_custom_property(
                    pos_id=pos_id,
                    order_id=order_id,
                    key="CANCELED_AFTER_PAID",
                    value="true",
                )

                if response.get("xml_response") is not None:
                    self.save_cancel_xml(
                        pos_id=pos_id,
                        order_id=order_id,
                        data=response.get("xml_response"),
                    )

            return response.get("status"), response.get("message")

    def _cancel_sat(
        self,
        pos_id,     # type: str
        order_id,   # type: str
    ):
        # type: (...) -> Tuple[bool, str]

        with FiscalDataRepository(self.mbcontext) as fiscal_repository:
            order_data = fiscal_repository.get_numero_sat_and_xml_request(order_id=order_id)
            response = self.fiscal_processor.cancel_sale(
                order_data=order_data,
                order_id=order_id,
            )
            if response.get("status") and response.get("xml_response", None) is not None:
                self.save_cancel_xml(pos_id, order_id, response.get("xml_response"))

            return response.get("status"), response.get("message")

    def save_cancel_xml(
            self,
            pos_id,     # type: str
            order_id,   # type: str
            data        # type: str
    ):
        # type: (...) -> None

        with FiscalDataRepository(self.mbcontext) as fiscal_repository:
            fiscal_repository.set_fiscal_xml_custom_property(
                pos_id=pos_id,
                xml_request=data,
                order_id=order_id,
                key='CANCELED_FISCAL_XML'
            )

    @staticmethod
    def _busca_parametros_sitef(order):
        # type: (eTree) -> (str, str, str)
        order_id = order.get("orderId")
        created_at = order.get("createdAt").replace("-", "").replace(":", "")
        data_fiscal = created_at[:8]
        hora_fiscal = created_at[9:15]

        return order_id, data_fiscal, hora_fiscal

    @staticmethod
    def _get_xml_sat(xml_request):
        try:
            xml = eTree.XML(base64.b64decode(xml_request))
            ide = xml.find('infCFe').find('ide')
            sat_key = xml.find('infCFe').attrib.get('Id')

            cpf_cnpj = xml.find('infCFe').find('dest') or ""
            if cpf_cnpj:
                if cpf_cnpj.find("CPF") is not None:
                    cpf_cnpj = cpf_cnpj.find("CPF").text
                elif cpf_cnpj.find("CNPJ") is not None:
                    cpf_cnpj = cpf_cnpj.find("CNPJ").text
                else:
                    cpf_cnpj = None

            date_time = "{0}{1}".format(ide.find('dEmi').text, ide.find('hEmi').text)
            date_time_reprint = datetime.strptime(date_time, "%Y%m%d%H%M%S").strftime("%d/%m/%y %H:%M:%S")

            total = xml.find('infCFe').find('total')

            customer_doc = cpf_cnpj if cpf_cnpj is not None else 'Nao informado'

            qr_code_values = [sat_key[3:],
                              date_time,
                              total.find('vCFe').text,
                              customer_doc,
                              ide.find('assinaturaQRCODE').text]

            qr_code = "|".join(qr_code_values)

            list_tags_xml = ['vICMS', 'vPIS', 'vCOFINS', 'vPISST', 'vCOFINSST']
            total_taxes = 0
            icms_total = total.find('ICMSTot')
            for x in list_tags_xml:
                try:
                    total_taxes += D(icms_total.find(x).text)
                except (TypeError, AttributeError):
                    pass
            total_taxes = round_half_away_from_zero(total_taxes, 2)

            tag_xml_request = eTree.XML("<XmlRequest>{0}</XmlRequest>".format(xml_request))
            tag_fiscal_data = "<FiscalData>"
            tag_fiscal_data += "<datetime>{0}</datetime>".format(date_time_reprint)
            tag_fiscal_data += "<satno>{0}</satno>".format(ide.find('nserieSAT').text)
            tag_fiscal_data += "<satkey>{0}</satkey>".format(sat_key[3:])
            tag_fiscal_data += "<CustomerCPF>{0}</CustomerCPF>".format(customer_doc)
            tag_fiscal_data += "<QRCode>{0}</QRCode>".format(qr_code)
            tag_fiscal_data += "</FiscalData>"

            return {"xml_request": tag_xml_request, "fiscal_data": tag_fiscal_data, "total_taxes": total_taxes}
        except Exception as ex:
            logger.error("Erro ao construir o xml - Exception %s", str(ex))
            return {"xml_request": None, "fiscal_data": None, "total_taxes": 0}

    def _get_xml_nfce(self, xml_request, xml_response):
        try:
            xml = remove_xml_namespace(base64.b64decode(xml_request))
            xml_resp = remove_xml_namespace(base64.b64decode(xml_response))

            xml_ws_version = self._get_xml_ws_version(xml_response=xml_resp)
            if xml_ws_version != self.versao_ws:
                error_message = "Versão do XML diferente da versão fiscal atual. Versão XML: {}; Versão Fiscal: {}"
                raise StopIteration(error_message.format(xml_ws_version, self.versao_ws))

            c_stat_xml_response = self._get_c_stat_code(xml_data=xml_resp)
            c_stat_xml_request = self._get_c_stat_code(xml_data=xml)
            if c_stat_xml_response == NfceCstatCode.duplicity and c_stat_xml_request == NfceCstatCode.authorized:
                xml_resp = xml

            nprot = xml_resp.find(".//infProt/nProt").text
            data_resp = xml_resp.find(".//infProt/dhRecbto").text

            if xml.find('.//NFe'):
                body = eTree.tostring(xml.find('.//NFe'))
            else:
                body = eTree.tostring(xml)

            version = xml.find(".//infNFe").attrib['versao']
            total = xml.find('.//ICMSTot')

            list_tags_xml = ['vICMS', 'vPIS', 'vCOFINS', 'vPISST', 'vCOFINSST']
            total_taxes = 0

            for tag in list_tags_xml:
                try:
                    total_taxes += D(total.find('.//{}'.format(tag)).text)
                except (TypeError, AttributeError):
                    pass

            total_taxes = round_half_away_from_zero(total_taxes, 2)

            tag_request = '<ns0:Envelope xmlns = "http://www.portalfiscal.inf.br/nfe" ' \
                          'xmlns:ns0 = "http://www.w3.org/2003/05/soap-envelope" ' \
                          'xmlns:ns1 = "http://www.portalfiscal.inf.br/nfe/wsdl/NfeAutorizacao" ' \
                          'xmlns:ns3 = "http://www.w3.org/2000/09/xmldsig#">'
            tag_request += '<ns0:Body><ns1:nfeDadosMsg>'
            tag_request += '<enviNFe versao = "{0}">'.format(version)
            tag_request += '{0}'.format(body)
            tag_request += '</enviNFe></ns1:nfeDadosMsg></ns0:Body></ns0:Envelope>'

            tag_reponse = '<ns0:Envelope xmlns="http://www.portalfiscal.inf.br/nfe" ' \
                          'xmlns:ns0="http://www.w3.org/2003/05/soap-envelope" ' \
                          'xmlns:ns1="http://www.portalfiscal.inf.br/nfe/wsdl/NfeRetAutorizacao">' \
                          '<ns0:Body><ns1:nfeRetAutorizacaoLoteResult>'
            tag_reponse += '<retConsReciNFe versao="{0}">'.format(version)
            tag_reponse += '<protNFe versao="{0}"><infProt>'.format(version)
            tag_reponse += '<dhRecbto>{0}</dhRecbto>'.format(str(data_resp))
            tag_reponse += '<nProt>{0}</nProt>'.format(str(nprot))

            re_search = r"<[A-z0-9]*?:?{0}>(.*?)</([A-z0-9]*?):?{0}>"
            if regex_search(re_search, eTree.tostring(xml_resp), key="cMsg"):
                message_code = regex_search(re_search, eTree.tostring(xml_resp), key="cMsg")
                message = regex_search(re_search, eTree.tostring(xml_resp), key="xMsg")
                if message_code == '200' and message not in [None, '', ' ']:
                    tag_reponse += "<cMsg>{}</cMsg>".format(message_code)
                    tag_reponse += "<xMsg>{}</xMsg>".format(message)

            tag_reponse += "</infProt></protNFe></retConsReciNFe></ns1:nfeRetAutorizacaoLoteResult></ns0:Body>"
            tag_reponse += "</ns0:Envelope>"

            return {
                "request_xml": eTree.XML(tag_request),
                "response_xml": eTree.XML(tag_reponse),
                "total_taxes": total_taxes,
            }
        except StopIteration:
            raise
        except Exception as ex:
            logger.error("Erro ao construir o xml - Exception %s", str(ex))
            return {"request_xml": None, "response_xml": None, "total_taxes": 0}

    @staticmethod
    def _get_xml_ws_version(
        xml_response,   # type: eTree.Element
    ):
        # type: (...) -> int

        tags = ["retConsReciNFe", "retEnviNFe", "retConsSitNFe"]
        for tag in tags:
            xml_resp = xml_response.find(".//{}".format(tag))
            if not xml_resp:
                continue

            return int(float(xml_resp.get('versao')))

        raise WsVersionNotFound()

    def _handle_re_sign_xml(self):
        with FiscalDataRepository(self.mbcontext) as fiscalrepository:
            logger.info("Starting XML Resign Event...")
            conn = None
            try:
                conn = DBDriver().open(self.mbcontext, service_name="FiscalPersistence")
                sql = """SELECT PosId, OrderId, XMLRequest
                          FROM fiscal.FiscalData
                          WHERE SentToNfce IN (555)
                          ORDER BY OrderId ASC LIMIT 500"""

                cursor = conn.select(sql)
                orders_with_error = 0
                orders_with_success = 0
                for row in cursor:
                    try:
                        pos_id, order_id, request = map(
                            row.get_entry,
                            ("PosId", "OrderId", "XMLRequest"),
                        )
                        logger.info("Resigning Order %s" % order_id)

                        if self.nf_type != "PAF":
                            try:
                                is_in_contingence = True
                                xml_decoded = base64.b64decode(request + "=" * ((4 - len(request) % 4) % 4))
                                req = remove_xml_namespace(xml_decoded)

                                dh_cont = req.find(".//infNFe/ide/dhCont")
                                if dh_cont is None:
                                    dh_cont = req.find(".//NFe/infNFe/ide/dhCont")

                                x_just = req.find(".//infNFe/ide/xJust")
                                if x_just is None:
                                    x_just = req.find(".//NFe/infNFe/ide/xJust")

                                if dh_cont is not None:
                                    dh_cont = dh_cont.text
                                    dh_cont = dh_cont[:-6]
                                    dh_cont = datetime.strptime(dh_cont, "%Y-%m-%dT%H:%M:%S")
                                if x_just is not None:
                                    x_just = x_just.text
                                if dh_cont is None and x_just is None:
                                    dh_cont = datetime.now()
                                    x_just = "Problemas de conexao com a SEFAZ"
                            except (Exception,):
                                dh_cont = datetime.now()
                                x_just = "Problemas de conexao com a SEFAZ"

                                logger.info("Order: {} - dh_cont: {} / xjust: {}".format(order_id, dh_cont, x_just))
                        else:
                            is_in_contingence = False
                            dh_cont = ""
                            x_just = ""
                            logger.info("Order: {} - Not in contingence".format(order_id))

                        order_taker = OrderTaker()
                        order = order_taker.get_order_picture(pos_id, order_id)
                        fiscalrepository.set_order_picture(order_id, base64.b64encode(order))
                        order = eTree.XML(order)

                        order.attrib["posId"] = pos_id

                        fiscal_data = self.fiscal_processor.nfce_request_builder.build_request(
                            order=order,
                            contingencia=is_in_contingence,
                            dh_contingencia=dh_cont,
                            just_contingencia=x_just,
                        )
                        index1 = fiscal_data[0].index("<NFe")
                        index2 = fiscal_data[0].index("</NFe>")
                        new_req = fiscal_data[0][index1:index2 + 6]
                        status = 0
                        xml_base64 = base64.b64encode(new_req)

                        fiscalrepository.set_nfce_sent_with_xml(order_id, xml_base64, status)
                        logger.info("XML Resign Order %s OK" % order_id)
                        orders_with_success += 1

                    except (Exception,):
                        orders_with_error += 1
                        logger.exception("Error to build XML of the order %s" % order_id)
                        continue
                if orders_with_error:
                    message = "Orders with success: {0}. Orders with error: {1}".format(
                        orders_with_success,
                        orders_with_error,
                    )
                    logger.error(message)
                    return False, message

            except (Exception,):
                logger.exception("Error to resign XMLs")
                return False, "Error to resign XMLs"
            finally:
                logger.info("Finish XML Resign Event...")
                if conn:
                    conn.close()

        return True, "Success to resigns XMLs. Number of orders resigns: {}".format(orders_with_success)

    def _handle_fiscal_xml_request(
        self,               # type: FiscalWrapperEventHandler
        data,               # type: str
        return_data=False,  # type: bool
    ):
        # type: (str, bool) -> Tuple[bool, str]

        conn = None
        base64_xml_response = None
        order_id = None
        try:
            conn = DBDriver().open(self.mbcontext, service_name="FiscalPersistence")
            sql = """SELECT OrderId, XMLRequest
                      FROM fiscal.FiscalData
                      WHERE OrderId=%s""" % data
            cursor = conn.select(sql)
            for row in cursor:
                order_id = row.get_entry(0)
                base64_xml_response = row.get_entry(1)
        except (Exception,):
            logger.exception("Erro ao buscar XMLRequest")
        finally:
            if conn:
                conn.close()
        if return_data:
            if order_id is None:
                return False, "Invalid Order ID {}".format(data)
            if base64_xml_response is None:
                return False, "No XML for order ID {}".format(order_id)
            return True, base64_xml_response
        else:
            if order_id is None:
                self.mbcontext.MB_EasyEvtSend(
                    subject="PosFiscalXmlError",
                    type="",
                    xml=";".join((
                        order_id,
                        "OrderId inválida ou inexistente no banco - order %s" % data,
                    )),
                )
            elif base64_xml_response is None:
                self.mbcontext.MB_EasyEvtSend(
                    subject="PosFiscalXmlError",
                    type="",
                    xml=";".join((
                        order_id,
                        "XML inexistente para order %s" % order_id,
                    )),
                )
            else:
                xml_response = base64.b64decode(base64_xml_response)
                compress_obj = zlib.compressobj(zlib.Z_BEST_COMPRESSION, zlib.DEFLATED, 31)
                zipped_xml = compress_obj.compress(xml_response)
                zipped_xml += compress_obj.flush()
                base64_zipped_xml = base64.b64encode(zipped_xml)
                self.mbcontext.MB_EasyEvtSend("PosFiscalXmlResponse", "", ";".join((order_id, base64_zipped_xml)))

    def _handle_check_if_fiscal_xml_is_created_in_folder(self):
        # type: () -> None
        try:
            logger.info("Starting -> _handle_check_if_fiscal_xml_is_created_in_folder")
            date_yesterday, date_yesterday_start, date_yesterday_end = self._get_yesterday_dates()
            fiscal_data_path = self._get_fiscal_xml_sent_path(date_yesterday)

            database_fiscal_data_number_list = self._get_fiscal_data_number_from_database_by_date(
                start_date=date_yesterday_start,
                end_date=date_yesterday_end,
            )

            self._generate_fiscal_xml_not_created(database_fiscal_data_number_list, fiscal_data_path)
            logger.info("Finished -> _handle_check_if_fiscal_xml_is_created_in_folder")
        except (Exception,):
            logger.exception("Error creating XML files")

    def _handle_retry_fiscal_orders_with_exception(self):
        # type: () -> None
        try:
            logger.info("Starting -> _handle_retry_fiscal_orders_with_exception")
            with FiscalDataRepository(self.mbcontext) as fiscal_repository:  # type: FiscalDataRepository
                fiscal_repository.retry_fiscal_orders_with_exception(self.period_to_retry_orders_with_exception)

            logger.info("Finished -> _handle_retry_fiscal_orders_with_exception")
        except (Exception,):
            logger.exception("Error changing status of fiscal orders with exception")

    def _get_fiscal_xml_sent_path(self, date_yesterday):
        fiscal_data_file = "{}/{}/{}".format(
            str(date_yesterday.year),
            str(date_yesterday.month).zfill(2),
            str(date_yesterday.day).zfill(2),
        )
        fiscal_data_path = os.path.join(self.fiscal_sent_dir, self.xml_enviados, fiscal_data_file)
        return fiscal_data_path

    def _generate_fiscal_xml_not_created(self, database_fiscal_data_number_list, fiscal_data_path):
        self._create_directory_if_not_exists(fiscal_data_path)

        self._verify_if_xml_in_folder_is_in_database_list(database_fiscal_data_number_list, fiscal_data_path)
        self._create_xml_if_it_is_not_in_folder(database_fiscal_data_number_list, fiscal_data_path)

    def _create_xml_if_it_is_not_in_folder(self, database_fiscal_data_number_list, fiscal_data_path):
        conn = None
        try:
            conn = DBDriver().open(self.mbcontext, service_name="FiscalPersistence")
            sql = """SELECT PosId, OrderId, XMLRequest, NumeroNota, NumeroSat FROM FiscalData
                     WHERE NumeroNota IN ({}) """.format("'" + "','".join(database_fiscal_data_number_list) + "'")

            cursor = conn.select(sql)
            for row in cursor:
                numero_nota, order_id, pos_id, xml_request, numero_sat = self._parse_database_info(row)

                inf_id, numero_serie = self._parse_xml_info(xml_request)

                fiscal_xml_path_to_save = self.create_fiscal_xml_path_to_save(inf_id, numero_serie,
                                                                              fiscal_data_path, numero_nota, order_id,
                                                                              pos_id, numero_sat)
                self._save_xml_file_in_folder(fiscal_xml_path_to_save, xml_request)
        except Exception as ex:
            logger.exception("Exception creating XML with error: {}".format(ex))
        finally:
            conn.close()

    def create_fiscal_xml_path_to_save(
        self,               # type: FiscalWrapperEventHandler
        inf_id,             # type: int
        numero_serie,       # type: str
        fiscal_data_path,   # type: str
        numero_nota,        # type: str
        order_id,           # type: str
        pos_id,             # type: str
        numero_sat,         # type: str
    ):
        # type: (...) -> str

        if self.nf_type in ["NFCE", "PAF"]:
            fiscal_xml_path_to_save = FiscalWrapperEventHandler._generate_nfce_xml_file_name(inf_id,
                                                                                             numero_nota,
                                                                                             order_id,
                                                                                             pos_id,
                                                                                             numero_serie,
                                                                                             fiscal_data_path)
        else:

            fiscal_xml_path_to_save = FiscalWrapperEventHandler._generate_sat_xml_file_name(fiscal_data_path,
                                                                                            numero_nota,
                                                                                            order_id,
                                                                                            pos_id,
                                                                                            numero_sat,
                                                                                            inf_id)
        return fiscal_xml_path_to_save

    @staticmethod
    def _generate_nfce_xml_file_name(infnfe_id, numero_nota, order_id, pos_id, numero_serie, fiscal_data_path):
        fiscal_xml_path_to_save = os.path.join(fiscal_data_path, "{0}_{1}_{2}_nfe_proc_pos{3}_{4}.xml".format(
            numero_serie.zfill(3),
            numero_nota.zfill(9),
            order_id.zfill(9),
            pos_id.zfill(2),
            infnfe_id))
        return fiscal_xml_path_to_save

    @staticmethod
    def _generate_sat_xml_file_name(fiscal_data_path, numero_nota, order_id, pos_id, numero_sat, infcfe_id):
        fiscal_xml_path_to_save = os.path.join(fiscal_data_path, "S{0}_{1}_{2}_response_pos{3}_{4}.xml".format(
            numero_sat[-2:],
            str(numero_nota).zfill(9),
            str(order_id).zfill(9),
            str(pos_id).zfill(2),
            infcfe_id))
        return fiscal_xml_path_to_save

    @staticmethod
    def _save_xml_file_in_folder(fiscal_xml_path_to_save, fiscal_xml_to_save):
        with open(fiscal_xml_path_to_save, "w+") as xml_to_save:
            xml_to_save.write(fiscal_xml_to_save)

    def _parse_xml_info(self, fiscal_xml_to_save):
        fiscal_xml_to_save = remove_xml_namespace(fiscal_xml_to_save)

        key_fiscal_inf = fiscal_xml_to_save.find(".//infCFe")
        key_data_fiscal_serie = fiscal_xml_to_save.find(".//nserieSAT")
        if self.nf_type in ("NFCE", "PAF"):
            key_fiscal_inf = fiscal_xml_to_save.find(".//infNFe")
            key_data_fiscal_serie = fiscal_xml_to_save.find(".//serie")

        data_fiscal_inf = key_fiscal_inf.attrib["Id"]
        data_fiscal_serie = key_data_fiscal_serie.text.zfill(3)

        return data_fiscal_inf, data_fiscal_serie

    @staticmethod
    def _parse_database_info(row):
        numero_nota = str(row.get_entry("NumeroNota")).zfill(9)
        order_id = str(row.get_entry("OrderId")).zfill(9)
        pos_id = str(row.get_entry("PosId")).zfill(2)
        xml_request = base64.b64decode(row.get_entry("XMLRequest"))
        numero_sat = str(row.get_entry("NumeroSat"))
        return numero_nota, order_id, pos_id, xml_request, numero_sat

    def _verify_if_xml_in_folder_is_in_database_list(self, database_fiscal_data_number_list, fiscal_data_path):
        for xml_file in glob.glob(os.path.join(fiscal_data_path, '*.xml')):
            try:
                xml_fiscal_number = self._get_xml_fiscal_number_from_xml_file(xml_file)

                if xml_fiscal_number in database_fiscal_data_number_list:
                    database_fiscal_data_number_list.remove(xml_fiscal_number)
                elif "procInut" not in xml_file:
                    self._move_xml_to_inconsistent_folder(fiscal_data_path, xml_file)

            except (Exception,):
                logger.exception("Exception on File: " + xml_file + "\nMoved to folder: " + self.xml_inconsistentes)
                self._move_xml_to_inconsistent_folder(fiscal_data_path, xml_file)

    def _get_xml_fiscal_number_from_xml_file(self, xml_file):
        xml = remove_xml_namespace(eTree.tostring(eTree.parse(xml_file).getroot()))
        key = xml.find(".//nNF") if self.nf_type == "NFCE" else xml.find(".//nCFe")
        return key.text if key is not None else None

    def _move_xml_to_inconsistent_folder(self, fiscal_data_path, xml_file):
        inconsistent_fiscal_data_path = fiscal_data_path.replace(self.xml_enviados, self.xml_inconsistentes)
        self._create_directory_if_not_exists(inconsistent_fiscal_data_path)
        xml_file_new_path = xml_file.replace(self.xml_enviados, self.xml_inconsistentes)
        shutil.move(xml_file, xml_file_new_path)
        logger.info("File: " + xml_file + "\nMoved to folder: " + self.xml_inconsistentes)

    @staticmethod
    def _create_directory_if_not_exists(data_path):
        try:
            if not os.path.exists(data_path):
                os.makedirs(data_path)
        except OSError:
            logger.error("Cannot create path: {}".format(data_path))

    @staticmethod
    def _get_yesterday_dates():
        date_yesterday = datetime.now() - timedelta(days=1)
        date_yesterday_start = datetime(date_yesterday.year, date_yesterday.month, date_yesterday.day, 0, 0, 0)
        date_yesterday_end = datetime(date_yesterday.year, date_yesterday.month, date_yesterday.day, 23, 59, 59)
        return date_yesterday, date_yesterday_start, date_yesterday_end

    def _get_fiscal_data_number_from_database_by_date(self, start_date, end_date):
        # type: (datetime, datetime) -> [unicode]
        fiscal_data_number_list = []
        conn = None

        try:
            conn = DBDriver().open(self.mbcontext, service_name="FiscalPersistence")
            sql = """SELECT NumeroNota, datetime(datanota,'unixepoch','localtime') AS data
                     FROM fiscal.FiscalData
                     WHERE data > '{}' AND data <= '{}'""".format(start_date, end_date)

            cursor = conn.select(sql)
            for row in cursor:
                fiscal_data_number_list.append(row.get_entry("NumeroNota"))
        except Exception as ex:
            logger.exception("Get fiscal data number list error: " + ex.message)
        finally:
            conn.close()

        return fiscal_data_number_list

    @staticmethod
    def _get_c_stat_code(
        xml_data,
    ):
        # type: (...) -> Optional[str]

        c_stat = xml_data.find(".//infProt/cStat")

        if c_stat is not None:
            return c_stat.text

        return None

    def _can_fiscalize_order(
        self,           # type: FiscalWrapperEventHandler
        total_gross,    # type: float
        tip_amount,     # type: float
    ):
        # type: (...) -> None

        self.fiscal_parameter_controller.can_fiscalize_order(
            total_gross=total_gross,
            tip_amount=tip_amount,
        )

    def _check_store_operator_with_sat(self):
        # type: (...) -> None

        if self.nf_type not in ("SAT", "MFE"):
            raise StoreNotOperatorWithSatException()



from typing import Optional


class CannotTotalOrderWithNotFilledMandatoryOption(Exception):

    def __init__(
        self,       # type: CannotTotalOrderWithNotFilledMandatoryOption
        part_code,  # type: int
        part_name,  # type: Optional[str]
    ):
        # type: (...) -> None
        super(CannotTotalOrderWithNotFilledMandatoryOption, self).__init__()

        self.part_code = part_code
        self.part_name = part_name



import argparse
import sys
from json import dumps
from os.path import abspath, basename, dirname, join, realpath
from platform import python_version
from typing import List, Optional
from unicodedata import unidata_version

import charset_normalizer.md as md_module
from charset_normalizer import from_fp
from charset_normalizer.models import CliDetectionResult
from charset_normalizer.version import __version__


def query_yes_no(question: str, default: str = "yes") -> bool:
    """Ask a yes/no question via input() and return their answer.

    "question" is a string that is presented to the user.
    "default" is the presumed answer if the user just hits <Enter>.
        It must be "yes" (the default), "no" or None (meaning
        an answer is required of the user).

    The "answer" return value is True for "yes" or False for "no".

    Credit goes to (c) https://stackoverflow.com/questions/3041986/apt-command-line-interface-like-yes-no-input
    """
    valid = {"yes": True, "y": True, "ye": True, "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower()
        if default is not None and choice == "":
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' " "(or 'y' or 'n').\n")


def cli_detect(argv: Optional[List[str]] = None) -> int:
    """
    CLI assistant using ARGV and ArgumentParser
    :param argv:
    :return: 0 if everything is fine, anything else equal trouble
    """
    parser = argparse.ArgumentParser(
        description="The Real First Universal Charset Detector. "
        "Discover originating encoding used on text file. "
        "Normalize text to unicode."
    )

    parser.add_argument(
        "files", type=argparse.FileType("rb"), nargs="+", help="File(s) to be analysed"
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        default=False,
        dest="verbose",
        help="Display complementary information about file if any. "
        "Stdout will contain logs about the detection process.",
    )
    parser.add_argument(
        "-a",
        "--with-alternative",
        action="store_true",
        default=False,
        dest="alternatives",
        help="Output complementary possibilities if any. Top-level JSON WILL be a list.",
    )
    parser.add_argument(
        "-n",
        "--normalize",
        action="store_true",
        default=False,
        dest="normalize",
        help="Permit to normalize input file. If not set, program does not write anything.",
    )
    parser.add_argument(
        "-m",
        "--minimal",
        action="store_true",
        default=False,
        dest="minimal",
        help="Only output the charset detected to STDOUT. Disabling JSON output.",
    )
    parser.add_argument(
        "-r",
        "--replace",
        action="store_true",
        default=False,
        dest="replace",
        help="Replace file when trying to normalize it instead of creating a new one.",
    )
    parser.add_argument(
        "-f",
        "--force",
        action="store_true",
        default=False,
        dest="force",
        help="Replace file without asking if you are sure, use this flag with caution.",
    )
    parser.add_argument(
        "-i",
        "--no-preemptive",
        action="store_true",
        default=False,
        dest="no_preemptive",
        help="Disable looking at a charset declaration to hint the detector.",
    )
    parser.add_argument(
        "-t",
        "--threshold",
        action="store",
        default=0.2,
        type=float,
        dest="threshold",
        help="Define a custom maximum amount of chaos allowed in decoded content. 0. <= chaos <= 1.",
    )
    parser.add_argument(
        "--version",
        action="version",
        version="Charset-Normalizer {} - Python {} - Unicode {} - SpeedUp {}".format(
            __version__,
            python_version(),
            unidata_version,
            "OFF" if md_module.__file__.lower().endswith(".py") else "ON",
        ),
        help="Show version information and exit.",
    )

    args = parser.parse_args(argv)

    if args.replace is True and args.normalize is False:
        if args.files:
            for my_file in args.files:
                my_file.close()
        print("Use --replace in addition of --normalize only.", file=sys.stderr)
        return 1

    if args.force is True and args.replace is False:
        if args.files:
            for my_file in args.files:
                my_file.close()
        print("Use --force in addition of --replace only.", file=sys.stderr)
        return 1

    if args.threshold < 0.0 or args.threshold > 1.0:
        if args.files:
            for my_file in args.files:
                my_file.close()
        print("--threshold VALUE should be between 0. AND 1.", file=sys.stderr)
        return 1

    x_ = []

    for my_file in args.files:
        matches = from_fp(
            my_file,
            threshold=args.threshold,
            explain=args.verbose,
            preemptive_behaviour=args.no_preemptive is False,
        )

        best_guess = matches.best()

        if best_guess is None:
            print(
                'Unable to identify originating encoding for "{}". {}'.format(
                    my_file.name,
                    (
                        "Maybe try increasing maximum amount of chaos."
                        if args.threshold < 1.0
                        else ""
                    ),
                ),
                file=sys.stderr,
            )
            x_.append(
                CliDetectionResult(
                    abspath(my_file.name),
                    None,
                    [],
                    [],
                    "Unknown",
                    [],
                    False,
                    1.0,
                    0.0,
                    None,
                    True,
                )
            )
        else:
            x_.append(
                CliDetectionResult(
                    abspath(my_file.name),
                    best_guess.encoding,
                    best_guess.encoding_aliases,
                    [
                        cp
                        for cp in best_guess.could_be_from_charset
                        if cp != best_guess.encoding
                    ],
                    best_guess.language,
                    best_guess.alphabets,
                    best_guess.bom,
                    best_guess.percent_chaos,
                    best_guess.percent_coherence,
                    None,
                    True,
                )
            )

            if len(matches) > 1 and args.alternatives:
                for el in matches:
                    if el != best_guess:
                        x_.append(
                            CliDetectionResult(
                                abspath(my_file.name),
                                el.encoding,
                                el.encoding_aliases,
                                [
                                    cp
                                    for cp in el.could_be_from_charset
                                    if cp != el.encoding
                                ],
                                el.language,
                                el.alphabets,
                                el.bom,
                                el.percent_chaos,
                                el.percent_coherence,
                                None,
                                False,
                            )
                        )

            if args.normalize is True:
                if best_guess.encoding.startswith("utf") is True:
                    print(
                        '"{}" file does not need to be normalized, as it already came from unicode.'.format(
                            my_file.name
                        ),
                        file=sys.stderr,
                    )
                    if my_file.closed is False:
                        my_file.close()
                    continue

                dir_path = dirname(realpath(my_file.name))
                file_name = basename(realpath(my_file.name))

                o_: List[str] = file_name.split(".")

                if args.replace is False:
                    o_.insert(-1, best_guess.encoding)
                    if my_file.closed is False:
                        my_file.close()
                elif (
                    args.force is False
                    and query_yes_no(
                        'Are you sure to normalize "{}" by replacing it ?'.format(
                            my_file.name
                        ),
                        "no",
                    )
                    is False
                ):
                    if my_file.closed is False:
                        my_file.close()
                    continue

                try:
                    x_[0].unicode_path = join(dir_path, ".".join(o_))

                    with open(x_[0].unicode_path, "wb") as fp:
                        fp.write(best_guess.output())
                except IOError as e:
                    print(str(e), file=sys.stderr)
                    if my_file.closed is False:
                        my_file.close()
                    return 2

        if my_file.closed is False:
            my_file.close()

    if args.minimal is False:
        print(
            dumps(
                [el.__dict__ for el in x_] if len(x_) > 1 else x_[0].__dict__,
                ensure_ascii=True,
                indent=4,
            )
        )
    else:
        for my_file in args.files:
            print(
                ", ".join(
                    [
                        el.encoding or "undefined"
                        for el in x_
                        if el.path == abspath(my_file.name)
                    ]
                )
            )

    return 0


if __name__ == "__main__":
    cli_detect()



import hashlib
from re import split
from typing import Any, Dict, Sequence
from urllib.parse import parse_qsl, unquote, urlparse

from . import contrib  # noqa:F401
from .compat import random
from .hotp import HOTP as HOTP
from .otp import OTP as OTP
from .totp import TOTP as TOTP


def random_base32(length: int = 32, chars: Sequence[str] = list("ABCDEFGHIJKLMNOPQRSTUVWXYZ234567")) -> str:
    # Note: the otpauth scheme DOES NOT use base32 padding for secret lengths not divisible by 8.
    # Some third-party tools have bugs when dealing with such secrets.
    # We might consider warning the user when generating a secret of length not divisible by 8.
    if length < 32:
        raise ValueError("Secrets should be at least 160 bits")

    return "".join(random.choice(chars) for _ in range(length))


def random_hex(length: int = 40, chars: Sequence[str] = list("ABCDEF0123456789")) -> str:
    if length < 40:
        raise ValueError("Secrets should be at least 160 bits")
    return random_base32(length=length, chars=chars)


def parse_uri(uri: str) -> OTP:
    """
    Parses the provisioning URI for the OTP; works for either TOTP or HOTP.

    See also:
        https://github.com/google/google-authenticator/wiki/Key-Uri-Format

    :param uri: the hotp/totp URI to parse
    :returns: OTP object
    """

    # Secret (to be filled in later)
    secret = None

    # Encoder (to be filled in later)
    encoder = None

    # Digits (to be filled in later)
    digits = None

    # Data we'll parse to the correct constructor
    otp_data: Dict[str, Any] = {}

    # Parse with URLlib
    parsed_uri = urlparse(unquote(uri))

    if parsed_uri.scheme != "otpauth":
        raise ValueError("Not an otpauth URI")

    # Parse issuer/accountname info
    accountinfo_parts = split(":|%3A", parsed_uri.path[1:], maxsplit=1)
    if len(accountinfo_parts) == 1:
        otp_data["name"] = accountinfo_parts[0]
    else:
        otp_data["issuer"] = accountinfo_parts[0]
        otp_data["name"] = accountinfo_parts[1]

    # Parse values
    for key, value in parse_qsl(parsed_uri.query):
        if key == "secret":
            secret = value
        elif key == "issuer":
            if "issuer" in otp_data and otp_data["issuer"] is not None and otp_data["issuer"] != value:
                raise ValueError("If issuer is specified in both label and parameters, it should be equal.")
            otp_data["issuer"] = value
        elif key == "algorithm":
            if value == "SHA1":
                otp_data["digest"] = hashlib.sha1
            elif value == "SHA256":
                otp_data["digest"] = hashlib.sha256
            elif value == "SHA512":
                otp_data["digest"] = hashlib.sha512
            else:
                raise ValueError("Invalid value for algorithm, must be SHA1, SHA256 or SHA512")
        elif key == "encoder":
            encoder = value
        elif key == "digits":
            digits = int(value)
            otp_data["digits"] = digits
        elif key == "period":
            otp_data["interval"] = int(value)
        elif key == "counter":
            otp_data["initial_count"] = int(value)
        elif key != "image":
            raise ValueError("{} is not a valid parameter".format(key))
    
    if encoder != "steam":
        if digits is not None and digits not in [6, 7, 8]:
            raise ValueError("Digits may only be 6, 7, or 8")
    
    if not secret:
        raise ValueError("No secret found in URI")

    # Create objects
    if encoder == "steam":
        return contrib.Steam(secret, **otp_data)
    if parsed_uri.netloc == "totp":
        return TOTP(secret, **otp_data)
    elif parsed_uri.netloc == "hotp":
        return HOTP(secret, **otp_data)

    raise ValueError("Not a supported OTP type")



"""Optional fixer that changes all unprefixed string literals "..." to b"...".

br'abcd' is a SyntaxError on Python 2 but valid on Python 3.
ur'abcd' is a SyntaxError on Python 3 but valid on Python 2.

"""
from __future__ import unicode_literals

import re
from lib2to3.pgen2 import token
from lib2to3 import fixer_base

_literal_re = re.compile(r"[^bBuUrR]?[\'\"]")

class FixBytes(fixer_base.BaseFix):
    BM_compatible = True
    PATTERN = "STRING"

    def transform(self, node, results):
        if node.type == token.STRING:
            if _literal_re.match(node.value):
                new = node.clone()
                new.value = u'b' + new.value
                return new



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: opentelemetry/proto/trace/v1/trace.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from opentelemetry.proto.common.v1 import (
    common_pb2 as opentelemetry_dot_proto_dot_common_dot_v1_dot_common__pb2,
)
from opentelemetry.proto.resource.v1 import (
    resource_pb2 as opentelemetry_dot_proto_dot_resource_dot_v1_dot_resource__pb2,
)


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n(opentelemetry/proto/trace/v1/trace.proto\x12\x1copentelemetry.proto.trace.v1\x1a*opentelemetry/proto/common/v1/common.proto\x1a.opentelemetry/proto/resource/v1/resource.proto"Q\n\nTracesData\x12\x43\n\x0eresource_spans\x18\x01 \x03(\x0b\x32+.opentelemetry.proto.trace.v1.ResourceSpans"\xa7\x01\n\rResourceSpans\x12;\n\x08resource\x18\x01 \x01(\x0b\x32).opentelemetry.proto.resource.v1.Resource\x12=\n\x0bscope_spans\x18\x02 \x03(\x0b\x32(.opentelemetry.proto.trace.v1.ScopeSpans\x12\x12\n\nschema_url\x18\x03 \x01(\tJ\x06\x08\xe8\x07\x10\xe9\x07"\x97\x01\n\nScopeSpans\x12\x42\n\x05scope\x18\x01 \x01(\x0b\x32\x33.opentelemetry.proto.common.v1.InstrumentationScope\x12\x31\n\x05spans\x18\x02 \x03(\x0b\x32".opentelemetry.proto.trace.v1.Span\x12\x12\n\nschema_url\x18\x03 \x01(\t"\x84\x08\n\x04Span\x12\x10\n\x08trace_id\x18\x01 \x01(\x0c\x12\x0f\n\x07span_id\x18\x02 \x01(\x0c\x12\x13\n\x0btrace_state\x18\x03 \x01(\t\x12\x16\n\x0eparent_span_id\x18\x04 \x01(\x0c\x12\r\n\x05\x66lags\x18\x10 \x01(\x07\x12\x0c\n\x04name\x18\x05 \x01(\t\x12\x39\n\x04kind\x18\x06 \x01(\x0e\x32+.opentelemetry.proto.trace.v1.Span.SpanKind\x12\x1c\n\x14start_time_unix_nano\x18\x07 \x01(\x06\x12\x1a\n\x12\x65nd_time_unix_nano\x18\x08 \x01(\x06\x12;\n\nattributes\x18\t \x03(\x0b\x32\'.opentelemetry.proto.common.v1.KeyValue\x12 \n\x18\x64ropped_attributes_count\x18\n \x01(\r\x12\x38\n\x06\x65vents\x18\x0b \x03(\x0b\x32(.opentelemetry.proto.trace.v1.Span.Event\x12\x1c\n\x14\x64ropped_events_count\x18\x0c \x01(\r\x12\x36\n\x05links\x18\r \x03(\x0b\x32\'.opentelemetry.proto.trace.v1.Span.Link\x12\x1b\n\x13\x64ropped_links_count\x18\x0e \x01(\r\x12\x34\n\x06status\x18\x0f \x01(\x0b\x32$.opentelemetry.proto.trace.v1.Status\x1a\x8c\x01\n\x05\x45vent\x12\x16\n\x0etime_unix_nano\x18\x01 \x01(\x06\x12\x0c\n\x04name\x18\x02 \x01(\t\x12;\n\nattributes\x18\x03 \x03(\x0b\x32\'.opentelemetry.proto.common.v1.KeyValue\x12 \n\x18\x64ropped_attributes_count\x18\x04 \x01(\r\x1a\xac\x01\n\x04Link\x12\x10\n\x08trace_id\x18\x01 \x01(\x0c\x12\x0f\n\x07span_id\x18\x02 \x01(\x0c\x12\x13\n\x0btrace_state\x18\x03 \x01(\t\x12;\n\nattributes\x18\x04 \x03(\x0b\x32\'.opentelemetry.proto.common.v1.KeyValue\x12 \n\x18\x64ropped_attributes_count\x18\x05 \x01(\r\x12\r\n\x05\x66lags\x18\x06 \x01(\x07"\x99\x01\n\x08SpanKind\x12\x19\n\x15SPAN_KIND_UNSPECIFIED\x10\x00\x12\x16\n\x12SPAN_KIND_INTERNAL\x10\x01\x12\x14\n\x10SPAN_KIND_SERVER\x10\x02\x12\x14\n\x10SPAN_KIND_CLIENT\x10\x03\x12\x16\n\x12SPAN_KIND_PRODUCER\x10\x04\x12\x16\n\x12SPAN_KIND_CONSUMER\x10\x05"\xae\x01\n\x06Status\x12\x0f\n\x07message\x18\x02 \x01(\t\x12=\n\x04\x63ode\x18\x03 \x01(\x0e\x32/.opentelemetry.proto.trace.v1.Status.StatusCode"N\n\nStatusCode\x12\x15\n\x11STATUS_CODE_UNSET\x10\x00\x12\x12\n\x0eSTATUS_CODE_OK\x10\x01\x12\x15\n\x11STATUS_CODE_ERROR\x10\x02J\x04\x08\x01\x10\x02*\x9c\x01\n\tSpanFlags\x12\x19\n\x15SPAN_FLAGS_DO_NOT_USE\x10\x00\x12 \n\x1bSPAN_FLAGS_TRACE_FLAGS_MASK\x10\xff\x01\x12*\n%SPAN_FLAGS_CONTEXT_HAS_IS_REMOTE_MASK\x10\x80\x02\x12&\n!SPAN_FLAGS_CONTEXT_IS_REMOTE_MASK\x10\x80\x04\x42w\n\x1fio.opentelemetry.proto.trace.v1B\nTraceProtoP\x01Z\'go.opentelemetry.io/proto/otlp/trace/v1\xaa\x02\x1cOpenTelemetry.Proto.Trace.V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "opentelemetry.proto.trace.v1.trace_pb2", _globals
)
if not _descriptor._USE_C_DESCRIPTORS:
    _globals["DESCRIPTOR"]._loaded_options = None
    _globals["DESCRIPTOR"]._serialized_options = (
        b"\n\037io.opentelemetry.proto.trace.v1B\nTraceProtoP\001Z'go.opentelemetry.io/proto/otlp/trace/v1\252\002\034OpenTelemetry.Proto.Trace.V1"
    )
    _globals["_SPANFLAGS"]._serialized_start = 1782
    _globals["_SPANFLAGS"]._serialized_end = 1938
    _globals["_TRACESDATA"]._serialized_start = 166
    _globals["_TRACESDATA"]._serialized_end = 247
    _globals["_RESOURCESPANS"]._serialized_start = 250
    _globals["_RESOURCESPANS"]._serialized_end = 417
    _globals["_SCOPESPANS"]._serialized_start = 420
    _globals["_SCOPESPANS"]._serialized_end = 571
    _globals["_SPAN"]._serialized_start = 574
    _globals["_SPAN"]._serialized_end = 1602
    _globals["_SPAN_EVENT"]._serialized_start = 1131
    _globals["_SPAN_EVENT"]._serialized_end = 1271
    _globals["_SPAN_LINK"]._serialized_start = 1274
    _globals["_SPAN_LINK"]._serialized_end = 1446
    _globals["_SPAN_SPANKIND"]._serialized_start = 1449
    _globals["_SPAN_SPANKIND"]._serialized_end = 1602
    _globals["_STATUS"]._serialized_start = 1605
    _globals["_STATUS"]._serialized_end = 1779
    _globals["_STATUS_STATUSCODE"]._serialized_start = 1695
    _globals["_STATUS_STATUSCODE"]._serialized_end = 1773
# @@protoc_insertion_point(module_scope)



from production.command import ChangeProdStateCommand
from production.model import ProductionOrder
from production.view import ProductionView
from typing import List


class CommandView(ProductionView):
    def __init__(self, name, command_processor, command, logger):
        super(CommandView, self).__init__(name, logger)

        self.command_processor = command_processor
        self.command = command

    def handle_order(self, orders, synch=False):
        # type: (List[ProductionOrder], bool) -> None

        for order in orders:
            production_command = ChangeProdStateCommand(order.order_id, None, self.command)
            self.command_processor.handle_command(production_command)

    def get_view_tags(self):
        return []



# types.py
# Copyright (C) 2005-2024 the SQLAlchemy authors and contributors
# <see AUTHORS file>
#
# This module is part of SQLAlchemy and is released under
# the MIT License: https://www.opensource.org/licenses/mit-license.php

"""Compatibility namespace for sqlalchemy.sql.types.

"""


from __future__ import annotations

from .sql.sqltypes import _Binary as _Binary
from .sql.sqltypes import ARRAY as ARRAY
from .sql.sqltypes import BIGINT as BIGINT
from .sql.sqltypes import BigInteger as BigInteger
from .sql.sqltypes import BINARY as BINARY
from .sql.sqltypes import BLOB as BLOB
from .sql.sqltypes import BOOLEAN as BOOLEAN
from .sql.sqltypes import Boolean as Boolean
from .sql.sqltypes import CHAR as CHAR
from .sql.sqltypes import CLOB as CLOB
from .sql.sqltypes import Concatenable as Concatenable
from .sql.sqltypes import DATE as DATE
from .sql.sqltypes import Date as Date
from .sql.sqltypes import DATETIME as DATETIME
from .sql.sqltypes import DateTime as DateTime
from .sql.sqltypes import DECIMAL as DECIMAL
from .sql.sqltypes import DOUBLE as DOUBLE
from .sql.sqltypes import Double as Double
from .sql.sqltypes import DOUBLE_PRECISION as DOUBLE_PRECISION
from .sql.sqltypes import Enum as Enum
from .sql.sqltypes import FLOAT as FLOAT
from .sql.sqltypes import Float as Float
from .sql.sqltypes import Indexable as Indexable
from .sql.sqltypes import INT as INT
from .sql.sqltypes import INTEGER as INTEGER
from .sql.sqltypes import Integer as Integer
from .sql.sqltypes import Interval as Interval
from .sql.sqltypes import JSON as JSON
from .sql.sqltypes import LargeBinary as LargeBinary
from .sql.sqltypes import MatchType as MatchType
from .sql.sqltypes import NCHAR as NCHAR
from .sql.sqltypes import NULLTYPE as NULLTYPE
from .sql.sqltypes import NullType as NullType
from .sql.sqltypes import NUMERIC as NUMERIC
from .sql.sqltypes import Numeric as Numeric
from .sql.sqltypes import NVARCHAR as NVARCHAR
from .sql.sqltypes import PickleType as PickleType
from .sql.sqltypes import REAL as REAL
from .sql.sqltypes import SchemaType as SchemaType
from .sql.sqltypes import SMALLINT as SMALLINT
from .sql.sqltypes import SmallInteger as SmallInteger
from .sql.sqltypes import String as String
from .sql.sqltypes import STRINGTYPE as STRINGTYPE
from .sql.sqltypes import TEXT as TEXT
from .sql.sqltypes import Text as Text
from .sql.sqltypes import TIME as TIME
from .sql.sqltypes import Time as Time
from .sql.sqltypes import TIMESTAMP as TIMESTAMP
from .sql.sqltypes import TupleType as TupleType
from .sql.sqltypes import Unicode as Unicode
from .sql.sqltypes import UnicodeText as UnicodeText
from .sql.sqltypes import UUID as UUID
from .sql.sqltypes import Uuid as Uuid
from .sql.sqltypes import VARBINARY as VARBINARY
from .sql.sqltypes import VARCHAR as VARCHAR
from .sql.type_api import adapt_type as adapt_type
from .sql.type_api import ExternalType as ExternalType
from .sql.type_api import to_instance as to_instance
from .sql.type_api import TypeDecorator as TypeDecorator
from .sql.type_api import TypeEngine as TypeEngine
from .sql.type_api import UserDefinedType as UserDefinedType
from .sql.type_api import Variant as Variant



"""
requests.compat
~~~~~~~~~~~~~~~

This module previously handled import compatibility issues
between Python 2 and Python 3. It remains for backwards
compatibility until the next major version.
"""

import importlib
import sys

# -------------------
# Character Detection
# -------------------


def _resolve_char_detection():
    """Find supported character detection libraries."""
    chardet = None
    for lib in ("chardet", "charset_normalizer"):
        if chardet is None:
            try:
                chardet = importlib.import_module(lib)
            except ImportError:
                pass
    return chardet


chardet = _resolve_char_detection()

# -------
# Pythons
# -------

# Syntax sugar.
_ver = sys.version_info

#: Python 2.x?
is_py2 = _ver[0] == 2

#: Python 3.x?
is_py3 = _ver[0] == 3

# json/simplejson module import resolution
has_simplejson = False
try:
    import simplejson as json

    has_simplejson = True
except ImportError:
    import json

if has_simplejson:
    from simplejson import JSONDecodeError
else:
    from json import JSONDecodeError

# Keep OrderedDict for backwards compatibility.
from collections import OrderedDict
from collections.abc import Callable, Mapping, MutableMapping
from http import cookiejar as cookielib
from http.cookies import Morsel
from io import StringIO

# --------------
# Legacy Imports
# --------------
from urllib.parse import (
    quote,
    quote_plus,
    unquote,
    unquote_plus,
    urldefrag,
    urlencode,
    urljoin,
    urlparse,
    urlsplit,
    urlunparse,
)
from urllib.request import (
    getproxies,
    getproxies_environment,
    parse_http_list,
    proxy_bypass,
    proxy_bypass_environment,
)

builtin_str = str
str = str
bytes = bytes
basestring = (str, bytes)
numeric_types = (int, float)
integer_types = (int,)



# -*- coding: utf-8 -*-

# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google/api/client.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import launch_stage_pb2 as google_dot_api_dot_launch__stage__pb2
from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x17google/api/client.proto\x12\ngoogle.api\x1a\x1dgoogle/api/launch_stage.proto\x1a google/protobuf/descriptor.proto\x1a\x1egoogle/protobuf/duration.proto"\xbe\x01\n\x16\x43ommonLanguageSettings\x12\x1e\n\x12reference_docs_uri\x18\x01 \x01(\tB\x02\x18\x01\x12:\n\x0c\x64\x65stinations\x18\x02 \x03(\x0e\x32$.google.api.ClientLibraryDestination\x12H\n\x1aselective_gapic_generation\x18\x03 \x01(\x0b\x32$.google.api.SelectiveGapicGeneration"\xfb\x03\n\x15\x43lientLibrarySettings\x12\x0f\n\x07version\x18\x01 \x01(\t\x12-\n\x0claunch_stage\x18\x02 \x01(\x0e\x32\x17.google.api.LaunchStage\x12\x1a\n\x12rest_numeric_enums\x18\x03 \x01(\x08\x12/\n\rjava_settings\x18\x15 \x01(\x0b\x32\x18.google.api.JavaSettings\x12-\n\x0c\x63pp_settings\x18\x16 \x01(\x0b\x32\x17.google.api.CppSettings\x12-\n\x0cphp_settings\x18\x17 \x01(\x0b\x32\x17.google.api.PhpSettings\x12\x33\n\x0fpython_settings\x18\x18 \x01(\x0b\x32\x1a.google.api.PythonSettings\x12/\n\rnode_settings\x18\x19 \x01(\x0b\x32\x18.google.api.NodeSettings\x12\x33\n\x0f\x64otnet_settings\x18\x1a \x01(\x0b\x32\x1a.google.api.DotnetSettings\x12/\n\rruby_settings\x18\x1b \x01(\x0b\x32\x18.google.api.RubySettings\x12+\n\x0bgo_settings\x18\x1c \x01(\x0b\x32\x16.google.api.GoSettings"\xa8\x03\n\nPublishing\x12\x33\n\x0fmethod_settings\x18\x02 \x03(\x0b\x32\x1a.google.api.MethodSettings\x12\x15\n\rnew_issue_uri\x18\x65 \x01(\t\x12\x19\n\x11\x64ocumentation_uri\x18\x66 \x01(\t\x12\x16\n\x0e\x61pi_short_name\x18g \x01(\t\x12\x14\n\x0cgithub_label\x18h \x01(\t\x12\x1e\n\x16\x63odeowner_github_teams\x18i \x03(\t\x12\x16\n\x0e\x64oc_tag_prefix\x18j \x01(\t\x12;\n\x0corganization\x18k \x01(\x0e\x32%.google.api.ClientLibraryOrganization\x12;\n\x10library_settings\x18m \x03(\x0b\x32!.google.api.ClientLibrarySettings\x12)\n!proto_reference_documentation_uri\x18n \x01(\t\x12(\n rest_reference_documentation_uri\x18o \x01(\t"\xe3\x01\n\x0cJavaSettings\x12\x17\n\x0flibrary_package\x18\x01 \x01(\t\x12L\n\x13service_class_names\x18\x02 \x03(\x0b\x32/.google.api.JavaSettings.ServiceClassNamesEntry\x12\x32\n\x06\x63ommon\x18\x03 \x01(\x0b\x32".google.api.CommonLanguageSettings\x1a\x38\n\x16ServiceClassNamesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"A\n\x0b\x43ppSettings\x12\x32\n\x06\x63ommon\x18\x01 \x01(\x0b\x32".google.api.CommonLanguageSettings"A\n\x0bPhpSettings\x12\x32\n\x06\x63ommon\x18\x01 \x01(\x0b\x32".google.api.CommonLanguageSettings"\xf4\x01\n\x0ePythonSettings\x12\x32\n\x06\x63ommon\x18\x01 \x01(\x0b\x32".google.api.CommonLanguageSettings\x12N\n\x15\x65xperimental_features\x18\x02 \x01(\x0b\x32/.google.api.PythonSettings.ExperimentalFeatures\x1a^\n\x14\x45xperimentalFeatures\x12\x1d\n\x15rest_async_io_enabled\x18\x01 \x01(\x08\x12\'\n\x1fprotobuf_pythonic_types_enabled\x18\x02 \x01(\x08"B\n\x0cNodeSettings\x12\x32\n\x06\x63ommon\x18\x01 \x01(\x0b\x32".google.api.CommonLanguageSettings"\xaa\x03\n\x0e\x44otnetSettings\x12\x32\n\x06\x63ommon\x18\x01 \x01(\x0b\x32".google.api.CommonLanguageSettings\x12I\n\x10renamed_services\x18\x02 \x03(\x0b\x32/.google.api.DotnetSettings.RenamedServicesEntry\x12K\n\x11renamed_resources\x18\x03 \x03(\x0b\x32\x30.google.api.DotnetSettings.RenamedResourcesEntry\x12\x19\n\x11ignored_resources\x18\x04 \x03(\t\x12 \n\x18\x66orced_namespace_aliases\x18\x05 \x03(\t\x12\x1e\n\x16handwritten_signatures\x18\x06 \x03(\t\x1a\x36\n\x14RenamedServicesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x37\n\x15RenamedResourcesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"B\n\x0cRubySettings\x12\x32\n\x06\x63ommon\x18\x01 \x01(\x0b\x32".google.api.CommonLanguageSettings"@\n\nGoSettings\x12\x32\n\x06\x63ommon\x18\x01 \x01(\x0b\x32".google.api.CommonLanguageSettings"\xcf\x02\n\x0eMethodSettings\x12\x10\n\x08selector\x18\x01 \x01(\t\x12<\n\x0clong_running\x18\x02 \x01(\x0b\x32&.google.api.MethodSettings.LongRunning\x12\x1d\n\x15\x61uto_populated_fields\x18\x03 \x03(\t\x1a\xcd\x01\n\x0bLongRunning\x12\x35\n\x12initial_poll_delay\x18\x01 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x1d\n\x15poll_delay_multiplier\x18\x02 \x01(\x02\x12\x31\n\x0emax_poll_delay\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x35\n\x12total_poll_timeout\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration"+\n\x18SelectiveGapicGeneration\x12\x0f\n\x07methods\x18\x01 \x03(\t*\xa3\x01\n\x19\x43lientLibraryOrganization\x12+\n\'CLIENT_LIBRARY_ORGANIZATION_UNSPECIFIED\x10\x00\x12\t\n\x05\x43LOUD\x10\x01\x12\x07\n\x03\x41\x44S\x10\x02\x12\n\n\x06PHOTOS\x10\x03\x12\x0f\n\x0bSTREET_VIEW\x10\x04\x12\x0c\n\x08SHOPPING\x10\x05\x12\x07\n\x03GEO\x10\x06\x12\x11\n\rGENERATIVE_AI\x10\x07*g\n\x18\x43lientLibraryDestination\x12*\n&CLIENT_LIBRARY_DESTINATION_UNSPECIFIED\x10\x00\x12\n\n\x06GITHUB\x10\n\x12\x13\n\x0fPACKAGE_MANAGER\x10\x14:9\n\x10method_signature\x12\x1e.google.protobuf.MethodOptions\x18\x9b\x08 \x03(\t:6\n\x0c\x64\x65\x66\x61ult_host\x12\x1f.google.protobuf.ServiceOptions\x18\x99\x08 \x01(\t:6\n\x0coauth_scopes\x12\x1f.google.protobuf.ServiceOptions\x18\x9a\x08 \x01(\t:8\n\x0b\x61pi_version\x12\x1f.google.protobuf.ServiceOptions\x18\xc1\xba\xab\xfa\x01 \x01(\tBi\n\x0e\x63om.google.apiB\x0b\x43lientProtoP\x01ZAgoogle.golang.org/genproto/googleapis/api/annotations;annotations\xa2\x02\x04GAPIb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "google.api.client_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b"\n\016com.google.apiB\013ClientProtoP\001ZAgoogle.golang.org/genproto/googleapis/api/annotations;annotations\242\002\004GAPI"
    _COMMONLANGUAGESETTINGS.fields_by_name["reference_docs_uri"]._options = None
    _COMMONLANGUAGESETTINGS.fields_by_name[
        "reference_docs_uri"
    ]._serialized_options = b"\030\001"
    _JAVASETTINGS_SERVICECLASSNAMESENTRY._options = None
    _JAVASETTINGS_SERVICECLASSNAMESENTRY._serialized_options = b"8\001"
    _DOTNETSETTINGS_RENAMEDSERVICESENTRY._options = None
    _DOTNETSETTINGS_RENAMEDSERVICESENTRY._serialized_options = b"8\001"
    _DOTNETSETTINGS_RENAMEDRESOURCESENTRY._options = None
    _DOTNETSETTINGS_RENAMEDRESOURCESENTRY._serialized_options = b"8\001"
    _globals["_CLIENTLIBRARYORGANIZATION"]._serialized_start = 2892
    _globals["_CLIENTLIBRARYORGANIZATION"]._serialized_end = 3055
    _globals["_CLIENTLIBRARYDESTINATION"]._serialized_start = 3057
    _globals["_CLIENTLIBRARYDESTINATION"]._serialized_end = 3160
    _globals["_COMMONLANGUAGESETTINGS"]._serialized_start = 137
    _globals["_COMMONLANGUAGESETTINGS"]._serialized_end = 327
    _globals["_CLIENTLIBRARYSETTINGS"]._serialized_start = 330
    _globals["_CLIENTLIBRARYSETTINGS"]._serialized_end = 837
    _globals["_PUBLISHING"]._serialized_start = 840
    _globals["_PUBLISHING"]._serialized_end = 1264
    _globals["_JAVASETTINGS"]._serialized_start = 1267
    _globals["_JAVASETTINGS"]._serialized_end = 1494
    _globals["_JAVASETTINGS_SERVICECLASSNAMESENTRY"]._serialized_start = 1438
    _globals["_JAVASETTINGS_SERVICECLASSNAMESENTRY"]._serialized_end = 1494
    _globals["_CPPSETTINGS"]._serialized_start = 1496
    _globals["_CPPSETTINGS"]._serialized_end = 1561
    _globals["_PHPSETTINGS"]._serialized_start = 1563
    _globals["_PHPSETTINGS"]._serialized_end = 1628
    _globals["_PYTHONSETTINGS"]._serialized_start = 1631
    _globals["_PYTHONSETTINGS"]._serialized_end = 1875
    _globals["_PYTHONSETTINGS_EXPERIMENTALFEATURES"]._serialized_start = 1781
    _globals["_PYTHONSETTINGS_EXPERIMENTALFEATURES"]._serialized_end = 1875
    _globals["_NODESETTINGS"]._serialized_start = 1877
    _globals["_NODESETTINGS"]._serialized_end = 1943
    _globals["_DOTNETSETTINGS"]._serialized_start = 1946
    _globals["_DOTNETSETTINGS"]._serialized_end = 2372
    _globals["_DOTNETSETTINGS_RENAMEDSERVICESENTRY"]._serialized_start = 2261
    _globals["_DOTNETSETTINGS_RENAMEDSERVICESENTRY"]._serialized_end = 2315
    _globals["_DOTNETSETTINGS_RENAMEDRESOURCESENTRY"]._serialized_start = 2317
    _globals["_DOTNETSETTINGS_RENAMEDRESOURCESENTRY"]._serialized_end = 2372
    _globals["_RUBYSETTINGS"]._serialized_start = 2374
    _globals["_RUBYSETTINGS"]._serialized_end = 2440
    _globals["_GOSETTINGS"]._serialized_start = 2442
    _globals["_GOSETTINGS"]._serialized_end = 2506
    _globals["_METHODSETTINGS"]._serialized_start = 2509
    _globals["_METHODSETTINGS"]._serialized_end = 2844
    _globals["_METHODSETTINGS_LONGRUNNING"]._serialized_start = 2639
    _globals["_METHODSETTINGS_LONGRUNNING"]._serialized_end = 2844
    _globals["_SELECTIVEGAPICGENERATION"]._serialized_start = 2846
    _globals["_SELECTIVEGAPICGENERATION"]._serialized_end = 2889
# @@protoc_insertion_point(module_scope)



# Use secrets module if available (Python version >= 3.6) per PEP 506
try:
    from secrets import SystemRandom  # type: ignore
except ImportError:
    from random import SystemRandom

random = SystemRandom()



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import enum


class SeverityNumber(enum.Enum):
    """Numerical value of severity.

    Smaller numerical values correspond to less severe events
    (such as debug events), larger numerical values correspond
    to more severe events (such as errors and critical events).

    See the `Log Data Model`_ spec for more info and how to map the
    severity from source format to OTLP Model.

    .. _Log Data Model: https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/logs/data-model.md#field-severitynumber
    """

    UNSPECIFIED = 0
    TRACE = 1
    TRACE2 = 2
    TRACE3 = 3
    TRACE4 = 4
    DEBUG = 5
    DEBUG2 = 6
    DEBUG3 = 7
    DEBUG4 = 8
    INFO = 9
    INFO2 = 10
    INFO3 = 11
    INFO4 = 12
    WARN = 13
    WARN2 = 14
    WARN3 = 15
    WARN4 = 16
    ERROR = 17
    ERROR2 = 18
    ERROR3 = 19
    ERROR4 = 20
    FATAL = 21
    FATAL2 = 22
    FATAL3 = 23
    FATAL4 = 24


_STD_TO_OTEL = {
    10: SeverityNumber.DEBUG,
    11: SeverityNumber.DEBUG2,
    12: SeverityNumber.DEBUG3,
    13: SeverityNumber.DEBUG4,
    14: SeverityNumber.DEBUG4,
    15: SeverityNumber.DEBUG4,
    16: SeverityNumber.DEBUG4,
    17: SeverityNumber.DEBUG4,
    18: SeverityNumber.DEBUG4,
    19: SeverityNumber.DEBUG4,
    20: SeverityNumber.INFO,
    21: SeverityNumber.INFO2,
    22: SeverityNumber.INFO3,
    23: SeverityNumber.INFO4,
    24: SeverityNumber.INFO4,
    25: SeverityNumber.INFO4,
    26: SeverityNumber.INFO4,
    27: SeverityNumber.INFO4,
    28: SeverityNumber.INFO4,
    29: SeverityNumber.INFO4,
    30: SeverityNumber.WARN,
    31: SeverityNumber.WARN2,
    32: SeverityNumber.WARN3,
    33: SeverityNumber.WARN4,
    34: SeverityNumber.WARN4,
    35: SeverityNumber.WARN4,
    36: SeverityNumber.WARN4,
    37: SeverityNumber.WARN4,
    38: SeverityNumber.WARN4,
    39: SeverityNumber.WARN4,
    40: SeverityNumber.ERROR,
    41: SeverityNumber.ERROR2,
    42: SeverityNumber.ERROR3,
    43: SeverityNumber.ERROR4,
    44: SeverityNumber.ERROR4,
    45: SeverityNumber.ERROR4,
    46: SeverityNumber.ERROR4,
    47: SeverityNumber.ERROR4,
    48: SeverityNumber.ERROR4,
    49: SeverityNumber.ERROR4,
    50: SeverityNumber.FATAL,
    51: SeverityNumber.FATAL2,
    52: SeverityNumber.FATAL3,
    53: SeverityNumber.FATAL4,
}


def std_to_otel(levelno: int) -> SeverityNumber:
    """
    Map python log levelno as defined in https://docs.python.org/3/library/logging.html#logging-levels
    to OTel log severity number as defined here: https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/logs/data-model.md#field-severitynumber
    """
    if levelno < 10:
        return SeverityNumber.UNSPECIFIED
    if levelno > 53:
        return SeverityNumber.FATAL4
    return _STD_TO_OTEL[levelno]



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for RPC Framework's Face interface."""

import collections

# stream is referenced from specification in this module.
from grpc.framework.common import cardinality
from grpc.framework.common import style
from grpc.framework.foundation import stream  # pylint: disable=unused-import
from grpc.framework.interfaces.face import face


class _MethodImplementation(
    face.MethodImplementation,
    collections.namedtuple(
        "_MethodImplementation",
        [
            "cardinality",
            "style",
            "unary_unary_inline",
            "unary_stream_inline",
            "stream_unary_inline",
            "stream_stream_inline",
            "unary_unary_event",
            "unary_stream_event",
            "stream_unary_event",
            "stream_stream_event",
        ],
    ),
):
    pass


def unary_unary_inline(behavior):
    """Creates an face.MethodImplementation for the given behavior.

    Args:
      behavior: The implementation of a unary-unary RPC method as a callable value
        that takes a request value and an face.ServicerContext object and
        returns a response value.

    Returns:
      An face.MethodImplementation derived from the given behavior.
    """
    return _MethodImplementation(
        cardinality.Cardinality.UNARY_UNARY,
        style.Service.INLINE,
        behavior,
        None,
        None,
        None,
        None,
        None,
        None,
        None,
    )


def unary_stream_inline(behavior):
    """Creates an face.MethodImplementation for the given behavior.

    Args:
      behavior: The implementation of a unary-stream RPC method as a callable
        value that takes a request value and an face.ServicerContext object and
        returns an iterator of response values.

    Returns:
      An face.MethodImplementation derived from the given behavior.
    """
    return _MethodImplementation(
        cardinality.Cardinality.UNARY_STREAM,
        style.Service.INLINE,
        None,
        behavior,
        None,
        None,
        None,
        None,
        None,
        None,
    )


def stream_unary_inline(behavior):
    """Creates an face.MethodImplementation for the given behavior.

    Args:
      behavior: The implementation of a stream-unary RPC method as a callable
        value that takes an iterator of request values and an
        face.ServicerContext object and returns a response value.

    Returns:
      An face.MethodImplementation derived from the given behavior.
    """
    return _MethodImplementation(
        cardinality.Cardinality.STREAM_UNARY,
        style.Service.INLINE,
        None,
        None,
        behavior,
        None,
        None,
        None,
        None,
        None,
    )


def stream_stream_inline(behavior):
    """Creates an face.MethodImplementation for the given behavior.

    Args:
      behavior: The implementation of a stream-stream RPC method as a callable
        value that takes an iterator of request values and an
        face.ServicerContext object and returns an iterator of response values.

    Returns:
      An face.MethodImplementation derived from the given behavior.
    """
    return _MethodImplementation(
        cardinality.Cardinality.STREAM_STREAM,
        style.Service.INLINE,
        None,
        None,
        None,
        behavior,
        None,
        None,
        None,
        None,
    )


def unary_unary_event(behavior):
    """Creates an face.MethodImplementation for the given behavior.

    Args:
      behavior: The implementation of a unary-unary RPC method as a callable
        value that takes a request value, a response callback to which to pass
        the response value of the RPC, and an face.ServicerContext.

    Returns:
      An face.MethodImplementation derived from the given behavior.
    """
    return _MethodImplementation(
        cardinality.Cardinality.UNARY_UNARY,
        style.Service.EVENT,
        None,
        None,
        None,
        None,
        behavior,
        None,
        None,
        None,
    )


def unary_stream_event(behavior):
    """Creates an face.MethodImplementation for the given behavior.

    Args:
      behavior: The implementation of a unary-stream RPC method as a callable
        value that takes a request value, a stream.Consumer to which to pass the
        response values of the RPC, and an face.ServicerContext.

    Returns:
      An face.MethodImplementation derived from the given behavior.
    """
    return _MethodImplementation(
        cardinality.Cardinality.UNARY_STREAM,
        style.Service.EVENT,
        None,
        None,
        None,
        None,
        None,
        behavior,
        None,
        None,
    )


def stream_unary_event(behavior):
    """Creates an face.MethodImplementation for the given behavior.

    Args:
      behavior: The implementation of a stream-unary RPC method as a callable
        value that takes a response callback to which to pass the response value
        of the RPC and an face.ServicerContext and returns a stream.Consumer to
        which the request values of the RPC should be passed.

    Returns:
      An face.MethodImplementation derived from the given behavior.
    """
    return _MethodImplementation(
        cardinality.Cardinality.STREAM_UNARY,
        style.Service.EVENT,
        None,
        None,
        None,
        None,
        None,
        None,
        behavior,
        None,
    )


def stream_stream_event(behavior):
    """Creates an face.MethodImplementation for the given behavior.

    Args:
      behavior: The implementation of a stream-stream RPC method as a callable
        value that takes a stream.Consumer to which to pass the response values
        of the RPC and an face.ServicerContext and returns a stream.Consumer to
        which the request values of the RPC should be passed.

    Returns:
      An face.MethodImplementation derived from the given behavior.
    """
    return _MethodImplementation(
        cardinality.Cardinality.STREAM_STREAM,
        style.Service.EVENT,
        None,
        None,
        None,
        None,
        None,
        None,
        None,
        behavior,
    )



from ._TokenCreator import TokenCreator
from ._TokenPriority import TokenPriority


class DefaultToken(object):
    TK_SYS_ACK = TokenCreator.create_token(TokenPriority.high, "01", "1")
    TK_SYS_NAK = TokenCreator.create_token(TokenPriority.high, "01", "2")
    TK_SYS_BAD_REQUEST = TokenCreator.create_token(TokenPriority.high, "01", "3")
    TK_SYS_APPLICATION_ERROR = TokenCreator.create_token(TokenPriority.high, "01", "4")

    TK_EVT_EVENT = TokenCreator.create_token(TokenPriority.high, "04", "1")

    TK_HTTP_MSG = TokenCreator.create_token(TokenPriority.high, "99", "1")
